@@ -5,7 +5,7 @@
  * libradius.h	Structures and prototypes
  *		for the radius library.
  *
- * Version:	$Id: libradius.h,v 1.151 2008/01/03 08:19:49 aland Exp $
+ * Version:	$Id$
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSIDH(libradius_h, "$Id: libradius.h,v 1.151 2008/01/03 08:19:49 aland Exp $")
+RCSIDH(libradius_h, "$Id$")
 
 #include <freeradius-devel/missing.h>
 
@@ -66,11 +66,11 @@ RCSIDH(libradius_h, "$Id: libradius.h,v 1.151 2008/01/03 08:19:49 aland Exp $")
 #  define VENDORPEC_USR		429
 #define VENDORPEC_LUCENT	4846
 #define VENDORPEC_STARENT	8164
-#  define DEBUG			if (librad_debug) printf
-#  define debug_pair(vp)	do { if (librad_debug) { \
-					putchar('\t'); \
-					vp_print(stdout, vp); \
-					putchar('\n'); \
+#  define DEBUG			if (fr_debug_flag && fr_log_fp) fr_printf_log
+#  define debug_pair(vp)	do { if (fr_debug_flag && fr_log_fp) { \
+					fputc('\t', fr_log_fp); \
+					vp_print(fr_log_fp, vp); \
+					fputc('\n', fr_log_fp); \
 				     } \
 				} while(0)
 #  define TAG_VALID(x)          ((x) > 0 && (x) < 0x20)
@@ -98,6 +98,9 @@ typedef struct attr_flags {
 	unsigned int		array : 1; /* pack multiples into 1 attr */
 	unsigned int		has_value : 1; /* has a value */
 	unsigned int		has_value_alias : 1; /* has a value alias */
+	unsigned int		has_tlv : 1; /* has sub attributes */
+	unsigned int		is_tlv : 1; /* is a sub attribute */
+	unsigned int		encoded : 1; /* has been put into packet */
 
 	int8_t			tag;	      /* tag for tunneled attributes */
 	uint8_t		        encrypt;      /* encryption method */
@@ -112,11 +115,11 @@ typedef struct attr_flags {
 #define FLAG_ENCRYPT_ASCEND_SECRET   (3)
 
 typedef struct dict_attr {
-	char			name[40];
 	int			attr;
 	int			type;
 	int			vendor;
         ATTR_FLAGS              flags;
+	char			name[1];
 } DICT_ATTR;
 
 typedef struct dict_value {
@@ -129,6 +132,7 @@ typedef struct dict_vendor {
 	int			vendorpec;
 	int			type; /* length of type data */
 	int			length;	/* length of length data */
+	int			flags;
 	char			name[1];
 } DICT_VENDOR;
 
@@ -139,10 +143,12 @@ typedef union value_pair_data {
 	struct in6_addr		ipv6addr;
 	uint32_t		date;
 	uint32_t		integer;
+	int32_t			sinteger;
 	uint8_t			filter[32];
 	uint8_t			ifid[8]; /* struct? */
 	uint8_t			ipv6prefix[18]; /* struct? */
      	uint8_t			ether[6];
+	uint8_t			*tlv;
 } VALUE_PAIR_DATA;
 
 typedef struct value_pair {
@@ -164,6 +170,8 @@ typedef struct value_pair {
 #define vp_ipv6prefix data.ipv6prefix
 #define vp_filter     data.filter
 #define vp_ether      data.ether
+#define vp_signed     data.sinteger
+#define vp_tlv	      data.tlv
 
 #if 0
 #define vp_ipaddr     data.ipaddr.s_addr
@@ -217,7 +225,7 @@ typedef struct radius_packet {
 /*
  *	Printing functions.
  */
-void		librad_safeprint(char *in, size_t inlen,
+void		fr_print_string(const char *in, size_t inlen,
 				 char *out, size_t outlen);
 int     	vp_prints_value(char *out, size_t outlen,
 				VALUE_PAIR *vp, int delimitst);
@@ -320,6 +328,7 @@ void		pairdelete(VALUE_PAIR **, int);
 void		pairadd(VALUE_PAIR **, VALUE_PAIR *);
 void            pairreplace(VALUE_PAIR **first, VALUE_PAIR *add);
 int		paircmp(VALUE_PAIR *check, VALUE_PAIR *data);
+VALUE_PAIR	*paircopyvp(const VALUE_PAIR *vp);
 VALUE_PAIR	*paircopy(VALUE_PAIR *vp);
 VALUE_PAIR	*paircopy2(VALUE_PAIR *vp, int attr);
 void		pairmove(VALUE_PAIR **to, VALUE_PAIR **from);
@@ -334,21 +343,27 @@ VALUE_PAIR     *readvp2(FILE *fp, int *pfiledone, const char *errprefix);
  *	Error functions.
  */
 #ifdef _LIBRADIUS
-void		librad_log(const char *, ...)
+void		fr_strerror_printf(const char *, ...)
 #ifdef __GNUC__
 		__attribute__ ((format (printf, 1, 2)))
 #endif
 ;
 #endif
-void		librad_perror(const char *, ...)
+void		fr_perror(const char *, ...)
+#ifdef __GNUC__
+		__attribute__ ((format (printf, 1, 2)))
+#endif
+;
+extern const char *fr_strerror(void);
+extern int	fr_dns_lookups;	/* 0 = no dns lookups */
+extern int	fr_debug_flag;	/* 0 = no debugging information */
+extern int	fr_max_attributes; /* per incoming packet */
+extern FILE	*fr_log_fp;
+void		fr_printf_log(const char *, ...)
 #ifdef __GNUC__
 		__attribute__ ((format (printf, 1, 2)))
 #endif
 ;
-extern char	librad_errstr[];
-extern int	librad_dodns;	/* 0 = no dns lookups */
-extern int	librad_debug;	/* 0 = no debugging information */
-extern int	librad_max_attributes; /* per incoming packet */
 
 /*
  *	Several handy miscellaneous functions.
@@ -374,7 +389,10 @@ int fr_ipaddr_cmp(const fr_ipaddr_t *a, const fr_ipaddr_t *b);
 
 int		ip_hton(const char *src, int af, fr_ipaddr_t *dst);
 const char	*ip_ntoh(const fr_ipaddr_t *src, char *dst, size_t cnt);
-
+int fr_ipaddr2sockaddr(const fr_ipaddr_t *ipaddr, int port,
+		       struct sockaddr_storage *sa, socklen_t *salen);
+int fr_sockaddr2ipaddr(const struct sockaddr_storage *sa, socklen_t salen,
+		       fr_ipaddr_t *ipaddr, int * port);
 
 
 #ifdef ASCEND_BINARY
