@@ -1,7 +1,7 @@
 /* modpriv.h: Stuff needed by both modules.c and modcall.c, but should not be
  * accessed from anywhere else.
  *
- * Version: $Id: modpriv.h,v 1.9 2008/06/05 15:56:18 aland Exp $ */
+ * Version: $Id$ */
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/modules.h>
 
@@ -30,6 +30,8 @@ typedef struct module_entry_t {
 	lt_dlhandle		handle;
 } module_entry_t;
 
+typedef struct fr_module_hup_t fr_module_hup_t;
+
 /*
  *	Per-instance data structure, to correlate the modules
  *	with the instance names (may NOT be the module names!),
@@ -42,7 +44,10 @@ typedef struct module_instance_t {
 #ifdef HAVE_PTHREAD_H
 	pthread_mutex_t		*mutex;
 #endif
-	void			*old_insthandle[16];
+	CONF_SECTION		*cs;
+	fr_module_hup_t	       	*mh;
 } module_instance_t;
 
-module_instance_t *find_module_instance(CONF_SECTION *, const char *instname);
+module_instance_t *find_module_instance(CONF_SECTION *, const char *instname,
+					int do_link);
+int module_hup_module(CONF_SECTION *cs, module_instance_t *node, time_t when);
