@@ -37,8 +37,6 @@ typedef struct auth_req REQUEST;
 
 #ifdef HAVE_PTHREAD_H
 #include	<pthread.h>
-typedef pthread_t child_pid_t;
-#define NO_SUCH_CHILD_PID (child_pid_t) (0)
 #endif
 
 #ifndef NDEBUG
@@ -192,7 +190,7 @@ struct auth_req {
 	request_data_t		*data;
 	RADCLIENT		*client;
 #ifdef HAVE_PTHREAD_H
-	child_pid_t    		child_pid;
+	pthread_t    		child_pid;
 #endif
 	time_t			timestamp;
 	int			number; /* internal server number */
@@ -355,6 +353,7 @@ typedef struct main_config_t {
 #define DEBUG	if(debug_flag)log_debug
 #define DEBUG2  if (debug_flag > 1)log_debug
 #define DEBUG3  if (debug_flag > 2)log_debug
+#define DEBUG4  if (debug_flag > 3)log_debug
 
 #if __GNUC__ >= 3
 #define RDEBUG(fmt, ...)   if(request && request->radlog) request->radlog(L_DBG, 1, request, fmt, ## __VA_ARGS__)
@@ -496,7 +495,7 @@ RADCLIENT	*client_findbynumber(const RADCLIENT_LIST *clients,
 RADCLIENT	*client_find_old(const fr_ipaddr_t *ipaddr);
 int		client_validate(RADCLIENT_LIST *clients, RADCLIENT *master,
 				RADCLIENT *c);
-RADCLIENT	*client_read(const char *filename, int in_server);
+RADCLIENT	*client_read(const char *filename, int in_server, int flag);
 
 
 /* files.c */
@@ -597,7 +596,7 @@ void listen_free(rad_listen_t **head);
 int listen_init(CONF_SECTION *cs, rad_listen_t **head);
 rad_listen_t *proxy_new_listener(void);
 RADCLIENT *client_listener_find(const rad_listen_t *listener,
-				const fr_ipaddr_t *ipaddr);
+				const fr_ipaddr_t *ipaddr, int src_port);
 #ifdef WITH_STATS
 RADCLIENT_LIST *listener_find_client_list(const fr_ipaddr_t *ipaddr,
 					  int port);
