@@ -1008,6 +1008,7 @@ int fr_dhcp_encode(RADIUS_PACKET *packet, RADIUS_PACKET *original)
 		uint8_t *plength, *pattr;
 
 		if (!IS_DHCP_ATTR(vp)) goto next;
+		if (vp->attribute == DHCP2ATTR(53)) goto next; /* already done */
 		if (((vp->attribute & 0xffff) > 255) &&
 		    (DHCP_BASE_ATTR(vp->attribute) != PW_DHCP_OPTION_82)) goto next;
 
@@ -1133,11 +1134,11 @@ int fr_dhcp_encode(RADIUS_PACKET *packet, RADIUS_PACKET *original)
 	if (dhcp->giaddr != htonl(INADDR_ANY)) {
 		packet->dst_ipaddr.ipaddr.ip4addr.s_addr = dhcp->giaddr;
 
-		/*
-		 *	Gateways send FROM 68 to 67, and we
-		 *	respond FROM 67 to 67... not to 68.
-		 */
-		packet->dst_port = original->dst_port;
+		if (dhcp->giaddr != htonl(INADDR_LOOPBACK)) {
+			packet->dst_port = original->dst_port;
+		} else {
+			packet->dst_port = original->src_port; /* debugging */
+		}
 
 	} else if (packet->code == PW_DHCP_NAK) {
 		packet->dst_ipaddr.ipaddr.ip4addr.s_addr = htonl(INADDR_BROADCAST);
