@@ -1,7 +1,7 @@
 /*
  * valuepair.c	Functions to handle VALUE_PAIRs
  *
- * Version:	$Id: valuepair.c,v 1.138 2008/03/13 11:17:13 aland Exp $
+ * Version:	$Id$
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Lesser General Public
@@ -21,7 +21,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: valuepair.c,v 1.138 2008/03/13 11:17:13 aland Exp $")
+RCSID("$Id$")
 
 #include	<freeradius-devel/libradius.h>
 
@@ -100,6 +100,7 @@ VALUE_PAIR *pairalloc(DICT_ATTR *da)
 		case PW_TYPE_INTEGER:
 		case PW_TYPE_IPADDR:
 		case PW_TYPE_DATE:
+		case PW_TYPE_SIGNED:
 			vp->length = 4;
 			break;
 
@@ -119,6 +120,12 @@ VALUE_PAIR *pairalloc(DICT_ATTR *da)
 			vp->length = sizeof(vp->vp_ether);
 			break;
 
+		case PW_TYPE_TLV:
+			vp->vp_tlv = NULL;
+			vp->length = 0;
+			break;
+
+		case PW_TYPE_COMBO_IP:
 		default:
 			vp->length = 0;
 			break;
@@ -138,7 +145,7 @@ VALUE_PAIR *paircreate(int attr, int type)
 
 	da = dict_attrbyvalue(attr);
 	if ((vp = pairalloc(da)) == NULL) {
-		librad_log("out of memory");
+		fr_strerror_printf("out of memory");
 		return NULL;
 	}
 	vp->operator = T_OP_EQ;
@@ -169,6 +176,7 @@ VALUE_PAIR *paircreate(int attr, int type)
  */
 void pairbasicfree(VALUE_PAIR *pair)
 {
+	if (pair->type == PW_TYPE_TLV) free(pair->vp_tlv);
 	/* clear the memory here */
 	memset(pair, 0, sizeof(*pair));
 	free(pair);
@@ -292,6 +300,38 @@ void pairreplace(VALUE_PAIR **first, VALUE_PAIR *replace)
 	*prev = replace;
 }
 
+
+/*
+ *	Copy just one VP.
+ */
+VALUE_PAIR *paircopyvp(const VALUE_PAIR *vp)
+{
+	size_t name_len;
+	VALUE_PAIR *n;
+	
+	if (!vp->flags.unknown_attr) {
+		name_len = 0;
+	} else {
+		name_len = FR_VP_NAME_PAD;
+	}
+	
+	if ((n = malloc(sizeof(*n) + name_len)) == NULL) {
+		fr_strerror_printf("out of memory");
+		return NULL;
+	}
+	memcpy(n, vp, sizeof(*n) + name_len);
+	n->next = NULL;
+
+	if ((n->type == PW_TYPE_TLV) &&
+	    (n->vp_tlv != NULL)) {
+		n->vp_tlv = malloc(n->length);
+		memcpy(n->vp_tlv, vp->vp_tlv, n->length);
+	}
+
+	return n;
+}
+
+
 /*
  *	Copy just a certain type of pairs.
  */
@@ -303,25 +343,13 @@ VALUE_PAIR *paircopy2(VALUE_PAIR *vp, int attr)
 	last = &first;
 
 	while (vp) {
-		size_t name_len;
-
 		if (attr >= 0 && vp->attribute != attr) {
 			vp = vp->next;
 			continue;
 		}
 
-		if (!vp->flags.unknown_attr) {
-			name_len = 0;
-		} else {
-			name_len = FR_VP_NAME_PAD;
-		}
-		
-		if ((n = malloc(sizeof(*n) + name_len)) == NULL) {
-			librad_log("out of memory");
-			return first;
-		}
-		memcpy(n, vp, sizeof(*n) + name_len);
-		n->next = NULL;
+		n = paircopyvp(vp);
+		if (!n) return first;
 		*last = n;
 		last = &n->next;
 		vp = vp->next;
@@ -805,8 +833,10 @@ VALUE_PAIR *pairparsevalue(VALUE_PAIR *vp, const char *value)
 	 *	Even for integers, dates and ip addresses we
 	 *	keep the original string in vp->vp_strvalue.
 	 */
-	strlcpy(vp->vp_strvalue, value, sizeof(vp->vp_strvalue));
-	vp->length = strlen(vp->vp_strvalue);
+	if (vp->type != PW_TYPE_TLV) {
+		strlcpy(vp->vp_strvalue, value, sizeof(vp->vp_strvalue));
+		vp->length = strlen(vp->vp_strvalue);
+	}
 
 	switch(vp->type) {
 		case PW_TYPE_STRING:
@@ -898,7 +928,7 @@ VALUE_PAIR *pairparsevalue(VALUE_PAIR *vp, const char *value)
 
 				if (ip_hton(cs, AF_INET, &ipaddr) < 0) {
 					free(s);
-					librad_log("Failed to find IP address for %s", cs);
+					fr_strerror_printf("Failed to find IP address for %s", cs);
 					return NULL;
 				}
 
@@ -919,7 +949,7 @@ VALUE_PAIR *pairparsevalue(VALUE_PAIR *vp, const char *value)
 			vp->vp_integer = (uint32_t) strtoul(value, &p, 10);
 			if (!*p) {
 				if (vp->vp_integer > 255) {
-					librad_log("Byte value \"%s\" is larger than 255", value);
+					fr_strerror_printf("Byte value \"%s\" is larger than 255", value);
 					return NULL;
 				}
 				vp->length = 1;
@@ -931,7 +961,7 @@ VALUE_PAIR *pairparsevalue(VALUE_PAIR *vp, const char *value)
 			 *	attribute.
 			 */
 			if ((dval = dict_valbyname(vp->attribute, value)) == NULL) {
-				librad_log("Unknown value %s for attribute %s",
+				fr_strerror_printf("Unknown value %s for attribute %s",
 					   value, vp->name);
 				return NULL;
 			}
@@ -946,7 +976,7 @@ VALUE_PAIR *pairparsevalue(VALUE_PAIR *vp, const char *value)
 			vp->vp_integer = (uint32_t) strtoul(value, &p, 10);
 			if (!*p) {
 				if (vp->vp_integer > 65535) {
-					librad_log("Byte value \"%s\" is larger than 65535", value);
+					fr_strerror_printf("Byte value \"%s\" is larger than 65535", value);
 					return NULL;
 				}
 				vp->length = 2;
@@ -958,7 +988,7 @@ VALUE_PAIR *pairparsevalue(VALUE_PAIR *vp, const char *value)
 			 *	attribute.
 			 */
 			if ((dval = dict_valbyname(vp->attribute, value)) == NULL) {
-				librad_log("Unknown value %s for attribute %s",
+				fr_strerror_printf("Unknown value %s for attribute %s",
 					   value, vp->name);
 				return NULL;
 			}
@@ -981,7 +1011,7 @@ VALUE_PAIR *pairparsevalue(VALUE_PAIR *vp, const char *value)
 			 *	attribute.
 			 */
 			if ((dval = dict_valbyname(vp->attribute, value)) == NULL) {
-				librad_log("Unknown value %s for attribute %s",
+				fr_strerror_printf("Unknown value %s for attribute %s",
 					   value, vp->name);
 				return NULL;
 			}
@@ -1000,7 +1030,7 @@ VALUE_PAIR *pairparsevalue(VALUE_PAIR *vp, const char *value)
 				time_t date;
 
 				if (gettime(value, &date) < 0) {
-					librad_log("failed to parse time string "
+					fr_strerror_printf("failed to parse time string "
 						   "\"%s\"", value);
 					return NULL;
 				}
@@ -1018,8 +1048,8 @@ VALUE_PAIR *pairparsevalue(VALUE_PAIR *vp, const char *value)
 			}
 
 		  	if (ascend_parse_filter(vp) < 0 ) {
-				librad_log("failed to parse Ascend binary attribute: %s",
-					   librad_errstr);
+				fr_strerror_printf("failed to parse Ascend binary attribute: %s",
+					   fr_strerror());
 				return NULL;
 			}
 			break;
@@ -1045,7 +1075,7 @@ VALUE_PAIR *pairparsevalue(VALUE_PAIR *vp, const char *value)
 				 *	die.
 				 */
 				if ((strlen(cp) & 0x01) != 0) {
-					librad_log("Hex string is not an even length string.");
+					fr_strerror_printf("Hex string is not an even length string.");
 					return NULL;
 				}
 
@@ -1055,7 +1085,7 @@ VALUE_PAIR *pairparsevalue(VALUE_PAIR *vp, const char *value)
 					unsigned int tmp;
 
 					if (sscanf(cp, "%02x", &tmp) != 1) {
-						librad_log("Non-hex characters at %c%c", cp[0], cp[1]);
+						fr_strerror_printf("Non-hex characters at %c%c", cp[0], cp[1]);
 						return NULL;
 					}
 
@@ -1068,7 +1098,7 @@ VALUE_PAIR *pairparsevalue(VALUE_PAIR *vp, const char *value)
 
 		case PW_TYPE_IFID:
 			if (ifid_aton(value, (void *) &vp->vp_ifid) == NULL) {
-				librad_log("failed to parse interface-id "
+				fr_strerror_printf("failed to parse interface-id "
 					   "string \"%s\"", value);
 				return NULL;
 			}
@@ -1077,7 +1107,7 @@ VALUE_PAIR *pairparsevalue(VALUE_PAIR *vp, const char *value)
 
 		case PW_TYPE_IPV6ADDR:
 			if (inet_pton(AF_INET6, value, &vp->vp_ipv6addr) <= 0) {
-				librad_log("failed to parse IPv6 address "
+				fr_strerror_printf("failed to parse IPv6 address "
 					   "string \"%s\"", value);
 				return NULL;
 			}
@@ -1087,7 +1117,7 @@ VALUE_PAIR *pairparsevalue(VALUE_PAIR *vp, const char *value)
 		case PW_TYPE_IPV6PREFIX:
 			p = strchr(value, '/');
 			if (!p || ((p - value) >= 256)) {
-				librad_log("invalid IPv6 prefix "
+				fr_strerror_printf("invalid IPv6 prefix "
 					   "string \"%s\"", value);
 				return NULL;
 			} else {
@@ -1098,14 +1128,14 @@ VALUE_PAIR *pairparsevalue(VALUE_PAIR *vp, const char *value)
 				buffer[p - value] = '\0';
 
 				if (inet_pton(AF_INET6, buffer, vp->vp_octets + 2) <= 0) {
-					librad_log("failed to parse IPv6 address "
+					fr_strerror_printf("failed to parse IPv6 address "
 						   "string \"%s\"", value);
 					return NULL;
 				}
 
 				prefix = strtoul(p + 1, &eptr, 10);
 				if ((prefix > 128) || *eptr) {
-					librad_log("failed to parse IPv6 address "
+					fr_strerror_printf("failed to parse IPv6 address "
 						   "string \"%s\"", value);
 					return NULL;
 				}
@@ -1137,7 +1167,7 @@ VALUE_PAIR *pairparsevalue(VALUE_PAIR *vp, const char *value)
 						c1 = c2 = NULL;
 					}
 					if (!c1 || !c2 || (length >= sizeof(vp->vp_ether))) {
-						librad_log("failed to parse Ethernet address \"%s\"", value);
+						fr_strerror_printf("failed to parse Ethernet address \"%s\"", value);
 						return NULL;
 					}
 					vp->vp_ether[length] = ((c1-hextab)<<4) + (c2-hextab);
@@ -1147,11 +1177,59 @@ VALUE_PAIR *pairparsevalue(VALUE_PAIR *vp, const char *value)
 			vp->length = 6;
 			break;
 
+		case PW_TYPE_COMBO_IP:
+			if (inet_pton(AF_INET6, value, vp->vp_strvalue) > 0) {
+				vp->type = PW_TYPE_IPV6ADDR;
+				vp->length = 16; /* length of IPv6 address */
+				vp->vp_strvalue[vp->length] = '\0';
+
+			} else {
+				fr_ipaddr_t ipaddr;
+
+				if (ip_hton(value, AF_INET, &ipaddr) < 0) {
+					fr_strerror_printf("Failed to find IPv4 address for %s", value);
+					return NULL;
+				}
+
+				vp->type = PW_TYPE_IPADDR;
+				vp->vp_ipaddr = ipaddr.ipaddr.ip4addr.s_addr;
+				vp->length = 4;
+			}
+			break;
+
+		case PW_TYPE_SIGNED: /* Damned code for 1 WiMAX attribute */
+			vp->vp_signed = (int32_t) strtol(value, &p, 10);
+			vp->length = 4;
+			break;
+
+		case PW_TYPE_TLV: /* don't use this! */
+			if (strncasecmp(value, "0x", 2) != 0) {
+				fr_strerror_printf("Invalid TLV specification");
+				return NULL;
+			}
+			length = strlen(value + 2) / 2;
+			if (vp->length < length) {
+				free(vp->vp_tlv);
+				vp->vp_tlv = NULL;
+			}
+			vp->vp_tlv = malloc(length);
+			if (!vp->vp_tlv) {
+				fr_strerror_printf("No memory");
+				return NULL;
+			}
+			if (fr_hex2bin(value + 2, vp->vp_tlv,
+				       length) != length) {
+				fr_strerror_printf("Invalid hex data in TLV");
+				return NULL;
+			}
+			vp->length = length;
+			break;
+
 			/*
 			 *  Anything else.
 			 */
 		default:
-			librad_log("unknown attribute type %d", vp->type);
+			fr_strerror_printf("unknown attribute type %d", vp->type);
 			return NULL;
 	}
 
@@ -1174,22 +1252,15 @@ static VALUE_PAIR *pairmake_any(const char *attribute, const char *value,
 	const char	*p = attribute;
 	char		*q;
 	VALUE_PAIR	*vp;
-	DICT_ATTR	*da;
 
 	/*
 	 *	Unknown attributes MUST be of type 'octets'
 	 */
 	if (value && (strncasecmp(value, "0x", 2) != 0)) {
-		librad_log("Invalid octet string \"%s\" for attribute name \"%s\"", value, attribute);
+		fr_strerror_printf("Invalid octet string \"%s\" for attribute name \"%s\"", value, attribute);
 		return NULL;
 	}
 
-	da = dict_attrbyname(attribute);
-	if (da) {
-		attr = da->attr;
-		goto raw;
-	}
-
 	attr = vendor = 0;
 
 	/*
@@ -1199,7 +1270,7 @@ static VALUE_PAIR *pairmake_any(const char *attribute, const char *value,
 		if (strncasecmp(p, "Vendor-", 7) == 0) {
 			vendor = (int) strtol(p + 7, &q, 10);
 			if ((vendor == 0) || (vendor > 65535)) {
-				librad_log("Invalid vendor value in attribute name \"%s\"", attribute);
+				fr_strerror_printf("Invalid vendor value in attribute name \"%s\"", attribute);
 				return NULL;
 			}
 
@@ -1211,12 +1282,12 @@ static VALUE_PAIR *pairmake_any(const char *attribute, const char *value,
 			q = strchr(p, '-');
 
 			if (!q) {
-				librad_log("Invalid vendor name in attribute name \"%s\"", attribute);
+				fr_strerror_printf("Invalid vendor name in attribute name \"%s\"", attribute);
 				return NULL;
 			}
 
 			if ((size_t) (q - p) >= sizeof(buffer)) {
-				librad_log("Vendor name too long in attribute name \"%s\"", attribute);
+				fr_strerror_printf("Vendor name too long in attribute name \"%s\"", attribute);
 				return NULL;
 			}
 
@@ -1225,7 +1296,7 @@ static VALUE_PAIR *pairmake_any(const char *attribute, const char *value,
 
 			vendor = dict_vendorbyname(buffer);
 			if (!vendor) {
-				librad_log("Unknown vendor name in attribute name \"%s\"", attribute);
+				fr_strerror_printf("Unknown vendor name in attribute name \"%s\"", attribute);
 				return NULL;
 			}
 
@@ -1233,7 +1304,7 @@ static VALUE_PAIR *pairmake_any(const char *attribute, const char *value,
 		}
 
 		if (*p != '-') {
-			librad_log("Invalid text following vendor definition in attribute name \"%s\"", attribute);
+			fr_strerror_printf("Invalid text following vendor definition in attribute name \"%s\"", attribute);
 			return NULL;
 		}
 		p++;
@@ -1243,7 +1314,7 @@ static VALUE_PAIR *pairmake_any(const char *attribute, const char *value,
 	 *	Attr-%d
 	 */
 	if (strncasecmp(p, "Attr-", 5) != 0) {
-		librad_log("Invalid format in attribute name \"%s\"", attribute);
+		fr_strerror_printf("Invalid format in attribute name \"%s\"", attribute);
 		return NULL;
 	}
 
@@ -1253,7 +1324,7 @@ static VALUE_PAIR *pairmake_any(const char *attribute, const char *value,
 	 *	Invalid, or trailing text after number.
 	 */
 	if ((attr == 0) || *q) {
-		librad_log("Invalid value in attribute name \"%s\"", attribute);
+		fr_strerror_printf("Invalid value in attribute name \"%s\"", attribute);
 		return NULL;
 	}
 
@@ -1266,7 +1337,7 @@ static VALUE_PAIR *pairmake_any(const char *attribute, const char *value,
 		if (!dv) {
 			if (attr > 255) {
 			attr_error:
-				librad_log("Invalid attribute number in attribute name \"%s\"", attribute);
+				fr_strerror_printf("Invalid attribute number in attribute name \"%s\"", attribute);
 				return NULL;
 			}
 
@@ -1284,21 +1355,20 @@ static VALUE_PAIR *pairmake_any(const char *attribute, const char *value,
 				break;
 
 			default:
-				librad_log("Internal sanity check failed");
+				fr_strerror_printf("Internal sanity check failed");
 				return NULL;
 		}
 	}
 
 	attr |= vendor << 16;
 
- raw:
 	/*
 	 *	We've now parsed the attribute properly, Let's create
 	 *	it.  This next stop also looks the attribute up in the
 	 *	dictionary, and creates the appropriate type for it.
 	 */
 	if ((vp = paircreate(attr, PW_TYPE_OCTETS)) == NULL) {
-		librad_log("out of memory");
+		fr_strerror_printf("out of memory");
 		return NULL;
 	}
 
@@ -1323,7 +1393,7 @@ static VALUE_PAIR *pairmake_any(const char *attribute, const char *value,
 	}
 
 	if (fr_hex2bin(value + 2, vp->vp_octets, size) != vp->length) {
-		librad_log("Invalid hex string");
+		fr_strerror_printf("Invalid hex string");
 		free(vp);
 		return NULL;
 	}
@@ -1360,10 +1430,8 @@ VALUE_PAIR *pairmake(const char *attribute, const char *value, int operator)
 	char            *tc, *ts;
 	signed char     tag;
 	int             found_tag;
-#ifdef HAVE_REGEX_H
-	int		res;
-	regex_t		cre;
-#endif
+	char		buffer[64];
+	const char	*attrname = attribute;
 
 	/*
 	 *    Check for tags in 'Attribute:Tag' format.
@@ -1373,11 +1441,15 @@ VALUE_PAIR *pairmake(const char *attribute, const char *value, int operator)
 
 	ts = strrchr(attribute, ':');
 	if (ts && !ts[1]) {
-		librad_log("Invalid tag for attribute %s", attribute);
+		fr_strerror_printf("Invalid tag for attribute %s", attribute);
 		return NULL;
 	}
 
 	if (ts && ts[1]) {
+		strlcpy(buffer, attribute, sizeof(buffer));
+		attrname = buffer;
+		ts = strrchr(attrname, ':');
+
 	         /* Colon found with something behind it */
 	         if (ts[1] == '*' && ts[2] == 0) {
 		         /* Wildcard tag for check items */
@@ -1390,7 +1462,7 @@ VALUE_PAIR *pairmake(const char *attribute, const char *value, int operator)
 				 *ts = 0;
 			 else tag = 0;
 		 } else {
-			 librad_log("Invalid tag for attribute %s", attribute);
+			 fr_strerror_printf("Invalid tag for attribute %s", attribute);
 			 return NULL;
 		 }
 		 found_tag = 1;
@@ -1400,17 +1472,12 @@ VALUE_PAIR *pairmake(const char *attribute, const char *value, int operator)
 	 *	It's not found in the dictionary, so we use
 	 *	another method to create the attribute.
 	 */
-	if ((da = dict_attrbyname(attribute)) == NULL) {
-		return pairmake_any(attribute, value, operator);
-	}
-
-	if (value && (value[0] == '0') && (value[1] == 'x') &&
-	    (da->type != PW_TYPE_OCTETS)) {
-		return pairmake_any(attribute, value, operator);
+	if ((da = dict_attrbyname(attrname)) == NULL) {
+		return pairmake_any(attrname, value, operator);
 	}
 
 	if ((vp = pairalloc(da)) == NULL) {
-		librad_log("out of memory");
+		fr_strerror_printf("out of memory");
 		return NULL;
 	}
 	vp->operator = (operator == 0) ? T_OP_EQ : operator;
@@ -1424,7 +1491,7 @@ VALUE_PAIR *pairmake(const char *attribute, const char *value, int operator)
 	        /* If we already found a tag, this is invalid */
 	        if(found_tag) {
 		        pairbasicfree(vp);
-			librad_log("Duplicate tag %s for attribute %s",
+			fr_strerror_printf("Duplicate tag %s for attribute %s",
 				   value, vp->name);
 			DEBUG("Duplicate tag %s for attribute %s\n",
 				   value, vp->name);
@@ -1472,36 +1539,12 @@ VALUE_PAIR *pairmake(const char *attribute, const char *value, int operator)
 		 */
 	case T_OP_REG_EQ:	/* =~ */
 	case T_OP_REG_NE:	/* !~ */
-		if (vp->type == PW_TYPE_INTEGER) {
-			return vp;
-		}
-#ifdef HAVE_REGEX_H
 		/*
-		 *	Regular expression match with no regular
-		 *	expression is wrong.
+		 *	If anything goes wrong, this is a run-time error,
+		 *	not a compile-time error.
 		 */
-		if (!value) {
-			pairfree(&vp);
-			return NULL;
-		}
+		return vp;
 
-		res = regcomp(&cre, value, REG_EXTENDED|REG_NOSUB);
-		if (res != 0) {
-			char	msg[128];
-
-			regerror(res, &cre, msg, sizeof(msg));
-			librad_log("Illegal regular expression in attribute: %s: %s",
-				vp->name, msg);
-			pairbasicfree(vp);
-			return NULL;
-		}
-		regfree(&cre);
-#else
-		librad_log("Regelar expressions not enabled in this build, error in attribute %s",
-				vp->name);
-		pairbasicfree(vp);
-		return NULL;
-#endif
 	}
 
 	/*
@@ -1551,7 +1594,7 @@ VALUE_PAIR *pairread(const char **ptr, FR_TOKEN *eol)
 {
 	char		buf[64];
 	char		attr[64];
-	char		value[512], *q;
+	char		value[1024], *q;
 	const char	*p;
 	FR_TOKEN	token, t, xlat;
 	VALUE_PAIR	*vp;
@@ -1564,13 +1607,13 @@ VALUE_PAIR *pairread(const char **ptr, FR_TOKEN *eol)
 
 	if (!*p) {
 		*eol = T_OP_INVALID;
-		librad_log("No token read where we expected an attribute name");
+		fr_strerror_printf("No token read where we expected an attribute name");
 		return NULL;
 	}
 
 	if (*p == '#') {
 		*eol = T_HASH;
-		librad_log("Read a comment instead of a token");
+		fr_strerror_printf("Read a comment instead of a token");
 		return NULL;
 	}
 
@@ -1585,7 +1628,7 @@ VALUE_PAIR *pairread(const char **ptr, FR_TOKEN *eol)
 
 	if (len == sizeof(attr)) {
 		*eol = T_OP_INVALID;
-		librad_log("Attribute name is too long");
+		fr_strerror_printf("Attribute name is too long");
 		return NULL;
 	}
 
@@ -1603,14 +1646,14 @@ VALUE_PAIR *pairread(const char **ptr, FR_TOKEN *eol)
 	/* Now we should have an operator here. */
 	token = gettoken(ptr, buf, sizeof(buf));
 	if (token < T_EQSTART || token > T_EQEND) {
-		librad_log("expecting operator");
+		fr_strerror_printf("expecting operator");
 		return NULL;
 	}
 
 	/* Read value.  Note that empty string values are allowed */
 	xlat = gettoken(ptr, value, sizeof(value));
 	if (xlat == T_EOL) {
-		librad_log("failed to get value");
+		fr_strerror_printf("failed to get value");
 		return NULL;
 	}
 
@@ -1620,7 +1663,7 @@ VALUE_PAIR *pairread(const char **ptr, FR_TOKEN *eol)
 	p = *ptr;
 	t = gettoken(&p, buf, sizeof(buf));
 	if (t != T_EOL && t != T_COMMA && t != T_HASH) {
-		librad_log("Expected end of line or comma");
+		fr_strerror_printf("Expected end of line or comma");
 		return NULL;
 	}
 
@@ -1645,7 +1688,7 @@ VALUE_PAIR *pairread(const char **ptr, FR_TOKEN *eol)
 		p = strchr(value, '%');
 		if (p && (p[1] == '{')) {
 			if (strlen(value) >= sizeof(vp->vp_strvalue)) {
-				librad_log("Value too long");
+				fr_strerror_printf("Value too long");
 				return NULL;
 			}
 			vp = pairmake(attr, NULL, token);
@@ -1658,17 +1701,51 @@ VALUE_PAIR *pairread(const char **ptr, FR_TOKEN *eol)
 			vp->flags.do_xlat = 1;
 			vp->length = 0;
 		} else {
+			/*
+			 *	Parse && escape it, as defined by the
+			 *	data type.
+			 */
 			vp = pairmake(attr, value, token);
+			if (!vp) {
+				*eol = T_OP_INVALID;
+				return NULL;
+			}
 		}
 		break;
 
+	case T_SINGLE_QUOTED_STRING:
+		vp = pairmake(attr, NULL, token);
+		if (!vp) {
+			*eol = T_OP_INVALID;
+			return NULL;
+		}
+
+		/*
+		 *	String and octet types get copied verbatim.
+		 */
+		if ((vp->type == PW_TYPE_STRING) ||
+		    (vp->type == PW_TYPE_OCTETS)) {
+			strlcpy(vp->vp_strvalue, value,
+				sizeof(vp->vp_strvalue));
+			vp->length = strlen(vp->vp_strvalue);
+
+			/*
+			 *	Everything else gets parsed: it's
+			 *	DATA, not a string!
+			 */
+		} else if (!pairparsevalue(vp, value)) {
+			pairfree(&vp);
+			*eol = T_OP_INVALID;
+			return NULL;
+		}
+		break;
 
 		/*
 		 *	Mark the pair to be allocated later.
 		 */
 	case T_BACK_QUOTED_STRING:
 		if (strlen(value) >= sizeof(vp->vp_strvalue)) {
-			librad_log("Value too long");
+			fr_strerror_printf("Value too long");
 			return NULL;
 		}
 
@@ -1773,7 +1850,7 @@ VALUE_PAIR *readvp2(FILE *fp, int *pfiledone, const char *errprefix)
 		last_token = userparse(buf, &vp);
 		if (!vp) {
 			if (last_token != T_EOL) {
-				librad_perror("%s", errprefix);
+				fr_perror("%s", errprefix);
 				error = 1;
 				break;
 			}
@@ -1832,7 +1909,7 @@ int paircmp(VALUE_PAIR *one, VALUE_PAIR *two)
 					  REG_EXTENDED);
 			if (compare != 0) {
 				regerror(compare, &reg, buffer, sizeof(buffer));
-				librad_log("Illegal regular expression in attribute: %s: %s",
+				fr_strerror_printf("Illegal regular expression in attribute: %s: %s",
 					   one->name, buffer);
 				return -1;
 			}
