@@ -146,9 +146,9 @@ static int fr_server_domain_socket(const char *path)
 
 	memset(&salocal, 0, sizeof(salocal));
         salocal.sun_family = AF_UNIX;
-	memcpy(salocal.sun_path, path, len); /* not zero terminated */
+	memcpy(salocal.sun_path, path, len + 1); /* SUN_LEN does strlen */
 	
-	socklen = sizeof(salocal.sun_family) + len;
+	socklen = SUN_LEN(&salocal);
 
 	/*
 	 *	Check the path.
