@@ -1,7 +1,7 @@
 /*
  * dhcp.c	DHCP processing.  Done poorly for now.
  *
- * Version:	$Id: dhcpd.c,v 1.1 2008/04/20 15:00:06 aland Exp $
+ * Version:	$Id$
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -43,7 +43,11 @@ static int dhcp_process(REQUEST *request)
 	 *	Look for Relay attribute, and forward it if so...
 	 */
 
-	switch (rcode) {
+	vp = pairfind(request->reply->vps, DHCP2ATTR(53)); /* DHCP-Message-Type */
+	if (vp) {
+		request->reply->code = vp->vp_integer;
+	}
+	else switch (rcode) {
 	case RLM_MODULE_OK:
 	case RLM_MODULE_UPDATED:
 		if (request->packet->code == (PW_DHCP_DISCOVER)) {
@@ -62,15 +66,11 @@ static int dhcp_process(REQUEST *request)
 	case RLM_MODULE_FAIL:
 	case RLM_MODULE_INVALID:
 	case RLM_MODULE_NOOP:
-		request->reply->code = PW_DHCP_NAK;
-		break;
-
 	case RLM_MODULE_NOTFOUND:
-		request->reply->code = PW_DHCP_DECLINE;
+		request->reply->code = PW_DHCP_NAK;
 		break;
 
 	case RLM_MODULE_HANDLED:
-		request->reply->code = PW_DHCP_NAK;
 		break;
 	}
 
@@ -121,18 +121,13 @@ static int dhcp_socket_recv(rad_listen_t *listener,
 
 	packet = fr_dhcp_recv(listener->fd);
 	if (!packet) {
-		radlog(L_ERR, "%s", librad_errstr);
+		radlog(L_ERR, "%s", fr_strerror());
 		return 0;
 	}
 
 	if ((client = client_listener_find(listener,
-					   &packet->src_ipaddr)) == NULL) {
-		char buffer[256];
-		radlog(L_ERR, "Ignoring request from unknown client %s port %d",
-		       inet_ntop(packet->src_ipaddr.af,
-				 &packet->src_ipaddr.ipaddr,
-				 buffer, sizeof(buffer)),
-		       packet->src_port);
+					   &packet->src_ipaddr,
+					   packet->src_port)) == NULL) {
 		rad_free(&packet);
 		return 0;
 	}
