@@ -1,7 +1,7 @@
 /*
  * evaluate.c	Evaluate complex conditions
  *
- * Version:	$Id: evaluate.c,v 1.38 2008/02/25 11:07:09 aland Exp $
+ * Version:	$Id$
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: evaluate.c,v 1.38 2008/02/25 11:07:09 aland Exp $")
+RCSID("$Id$")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/modules.h>
@@ -48,6 +48,8 @@ RCSID("$Id: evaluate.c,v 1.38 2008/02/25 11:07:09 aland Exp $")
 #endif
 
 
+#ifdef WITH_UNLANG
+
 static int all_digits(const char *string)
 {
 	const char *p = string;
@@ -59,14 +61,6 @@ static int all_digits(const char *string)
 	return (*p == '\0');
 }
 
-#ifndef NDEBUG
-#ifndef DEBUG4
-#define DEBUG4  if (debug_flag > 4)log_debug
-#endif
-#else
-#define DEBUG4 if (0) log_debug
-#endif
-
 static const char *filler = "????????????????????????????????????????????????????????????????";
 
 static const char *expand_string(char *buffer, size_t sizeof_buffer,
@@ -233,6 +227,7 @@ static int radius_get_vp(REQUEST *request, const char *name, VALUE_PAIR **vp_p)
 		vp_name += 6;
 		vps = myrequest->reply->vps;
 
+#ifdef WITH_PROXY
 	} else if (strncmp(vp_name, "proxy-request:", 14) == 0) {
 		vp_name += 14;
 		if (request->proxy) vps = myrequest->proxy->vps;
@@ -240,6 +235,7 @@ static int radius_get_vp(REQUEST *request, const char *name, VALUE_PAIR **vp_p)
 	} else if (strncmp(vp_name, "proxy-reply:", 12) == 0) {
 		vp_name += 12;
 		if (request->proxy_reply) vps = myrequest->proxy_reply->vps;
+#endif
 
 	} else if (strncmp(vp_name, "config:", 7) == 0) {
 		vp_name += 7;
@@ -275,7 +271,11 @@ static int radius_do_cmp(REQUEST *request, int *presult,
 	int result;
 	int lint, rint;
 	VALUE_PAIR *vp = NULL;
+#ifdef HAVE_REGEX_H
 	char buffer[1024];
+#else
+	cflags = cflags;	/* -Wunused */
+#endif
 
 	rt = rt;		/* -Wunused */
 
@@ -311,7 +311,25 @@ static int radius_do_cmp(REQUEST *request, int *presult,
 			}
 
 			if (!vp) {
-				DEBUG2("    (Attribute %s was not found)",
+				DICT_ATTR *da;
+				
+				/*
+				 *	The attribute on the LHS may
+				 *	have been a dynamically
+				 *	registered callback.  i.e. it
+				 *	doesn't exist as a VALUE_PAIR.
+				 *	If so, try looking for it.
+				 */
+				da = dict_attrbyname(pleft);
+				if (da && radius_find_compare(da->attr)) {
+					VALUE_PAIR *check = pairmake(pleft, pright, token);
+					*presult = (radius_callback_compare(request, NULL, check, NULL, NULL) == 0);
+					pairfree(&check);
+					if (*presult)  return TRUE;
+					return FALSE;
+				}
+				
+				RDEBUG2("    (Attribute %s was not found)",
 				       pleft);
 				return FALSE;
 			}
@@ -331,30 +349,32 @@ static int radius_do_cmp(REQUEST *request, int *presult,
 
 			memcpy(&myvp, vp, sizeof(myvp));
 			if (!pairparsevalue(&myvp, pright)) {
-				DEBUG2("Failed parsing \"%s\": %s",
-				       pright, librad_errstr);
+				RDEBUG2("Failed parsing \"%s\": %s",
+				       pright, fr_strerror());
 				return FALSE;
 			}
 
 			myvp.operator = token;
 			*presult = paircmp(&myvp, vp);
 			return TRUE;
-		} /* else it's not a attribute in the dictionary */
+		} /* else it's not a VP in a list */
 	}
 
+#ifdef HAVE_REGEX_H
 	do_checks:
+#endif
 	switch (token) {
 	case T_OP_GE:
 	case T_OP_GT:
 	case T_OP_LE:
 	case T_OP_LT:
 		if (!all_digits(pright)) {
-			DEBUG2("    (Right field is not a number at: %s)", pright);
+			RDEBUG2("    (Right field is not a number at: %s)", pright);
 			return FALSE;
 		}
 		rint = atoi(pright);
 		if (!all_digits(pleft)) {
-			DEBUG2("    (Left field is not a number at: %s)", pleft);
+			RDEBUG2("    (Left field is not a number at: %s)", pleft);
 			return FALSE;
 		}
 		lint = atoi(pleft);
@@ -413,7 +433,17 @@ static int radius_do_cmp(REQUEST *request, int *presult,
 		/*
 		 *	Include substring matches.
 		 */
-		regcomp(&reg, pright, cflags);
+		compare = regcomp(&reg, pright, cflags);
+		if (compare != 0) {
+			if (debug_flag) {
+				char errbuf[128];
+
+				regerror(compare, &reg, errbuf, sizeof(errbuf));
+				DEBUG("ERROR: Failed compiling regular expression: %s", errbuf);
+			}
+			return FALSE;
+		}
+
 		compare = regexec(&reg, pleft,
 				  REQUEST_MAX_REGEX + 1,
 				  rxmatch, 0);
@@ -427,6 +457,7 @@ static int radius_do_cmp(REQUEST *request, int *presult,
 
 			free(request_data_get(request, request,
 					      REQUEST_DATA_REGEX | i));
+
 			/*
 			 *	No %{i}, skip it.
 			 *	We MAY have %{2} without %{1}.
@@ -434,21 +465,13 @@ static int radius_do_cmp(REQUEST *request, int *presult,
 			if (rxmatch[i].rm_so == -1) continue;
 			
 			/*
-			 *	Copy substring into buffer.
+			 *	Copy substring into allocated buffer
 			 */
-			memcpy(buffer, pleft + rxmatch[i].rm_so,
+			r = rad_malloc(rxmatch[i].rm_eo -rxmatch[i].rm_so + 1);
+			memcpy(r, pleft + rxmatch[i].rm_so,
 			       rxmatch[i].rm_eo - rxmatch[i].rm_so);
-			buffer[rxmatch[i].rm_eo - rxmatch[i].rm_so] = '\0';
-			
-			/*
-			 *	Copy substring, and add it to
-			 *	the request.
-			 *
-			 *	Note that we don't check
-			 *	for out of memory, which is
-			 *	the only error we can get...
-			 */
-			r = strdup(buffer);
+			r[rxmatch[i].rm_eo - rxmatch[i].rm_so] = '\0';
+
 			request_data_add(request, request,
 					 REQUEST_DATA_REGEX | i,
 					 r, free);
@@ -465,7 +488,17 @@ static int radius_do_cmp(REQUEST *request, int *presult,
 		/*
 		 *	Include substring matches.
 		 */
-		regcomp(&reg, pright, cflags);
+		compare = regcomp(&reg, pright, cflags);
+		if (compare != 0) {
+			if (debug_flag) {
+				char errbuf[128];
+
+				regerror(compare, &reg, errbuf, sizeof(errbuf));
+				DEBUG("ERROR: Failed compiling regular expression: %s", errbuf);
+			}
+			return FALSE;
+		}
+
 		compare = regexec(&reg, pleft,
 				  REQUEST_MAX_REGEX + 1,
 				  rxmatch, 0);
@@ -477,7 +510,7 @@ static int radius_do_cmp(REQUEST *request, int *presult,
 #endif
 		
 	default:
-		DEBUG4(">>> NOT IMPLEMENTED %d", token);
+		RDEBUG4(">>> NOT IMPLEMENTED %d", token);
 		result = FALSE;
 		break;
 	}
@@ -493,26 +526,25 @@ int radius_evaluate_condition(REQUEST *request, int modreturn, int depth,
 	int result = TRUE;
 	int invert = FALSE;
 	int evaluate_next_condition = evaluate_it;
-	const char *p = *ptr;
+	const char *p;
 	const char *q, *start;
 	FR_TOKEN token, lt, rt;
 	char left[1024], right[1024], comp[4];
 	const char *pleft, *pright;
 	char  xleft[1024], xright[1024];
-#ifdef HAVE_REGEX_H
 	int cflags = 0;
-#endif
-
+	
 	if (!ptr || !*ptr || (depth >= 64)) {
 		radlog(L_ERR, "Internal sanity check failed in evaluate condition");
 		return FALSE;
 	}
 
+	p =  *ptr;
 	while (*p) {
 		while ((*p == ' ') || (*p == '\t')) p++;
 
 		if (*p == '!') {
-			DEBUG4(">>> INVERT");
+			RDEBUG4(">>> INVERT");
 			invert = TRUE;
 			p++;
 		}
@@ -527,7 +559,7 @@ int radius_evaluate_condition(REQUEST *request, int modreturn, int depth,
 			 *	Evaluate the condition, bailing out on
 			 *	parse error.
 			 */
-			DEBUG4(">>> CALLING EVALUATE %s", end);
+			RDEBUG4(">>> CALLING EVALUATE %s", end);
 			if (!radius_evaluate_condition(request, modreturn,
 						       depth + 1, &end,
 						       evaluate_next_condition,
@@ -536,7 +568,7 @@ int radius_evaluate_condition(REQUEST *request, int modreturn, int depth,
 			}
 
 			if (invert && evaluate_next_condition) {
-				DEBUG2("%.*s Converting !%s -> %s",
+				RDEBUG2("%.*s Converting !%s -> %s",
 				       depth, filler,
 				       (result != FALSE) ? "TRUE" : "FALSE",
 				       (result == FALSE) ? "TRUE" : "FALSE");
@@ -551,7 +583,7 @@ int radius_evaluate_condition(REQUEST *request, int modreturn, int depth,
 			 *	condition
 			 */
 			p = end;
-			DEBUG4(">>> EVALUATE RETURNED ::%s::", end);
+			RDEBUG4(">>> EVALUATE RETURNED ::%s::", end);
 			
 			if (!((*p == ')') || (*p == '!') ||
 			      ((p[0] == '&') && (p[1] == '&')) ||
@@ -569,7 +601,7 @@ int radius_evaluate_condition(REQUEST *request, int modreturn, int depth,
 			 *	EOL.  It's OK.
 			 */
 			if (!*p) {
-				DEBUG4(">>> AT EOL");
+				RDEBUG4(">>> AT EOL");
 				*ptr = p;
 				*presult = result;
 				return TRUE;
@@ -599,7 +631,7 @@ int radius_evaluate_condition(REQUEST *request, int modreturn, int depth,
 				p += 2;
 
 			} else if (*p == ')') {
-				DEBUG4(">>> CLOSING BRACE");
+				RDEBUG4(">>> CLOSING BRACE");
 				*ptr = p;
 				*presult = result;
 				return TRUE;
@@ -620,7 +652,7 @@ int radius_evaluate_condition(REQUEST *request, int modreturn, int depth,
 		 */
 		if ((*p == '(') || (p[0] == '!')) continue;
 
-		DEBUG4(">>> LOOKING AT %s", p);
+		RDEBUG4(">>> LOOKING AT %s", p);
 		start = p;
 
 		/*
@@ -731,7 +763,7 @@ int radius_evaluate_condition(REQUEST *request, int modreturn, int depth,
 			}
 		}
 		
-		DEBUG4(">>> %d:%s %d %d:%s",
+		RDEBUG4(">>> %d:%s %d %d:%s",
 		       lt, pleft, token, rt, pright);
 		
 	do_cmp:
@@ -744,25 +776,25 @@ int radius_evaluate_condition(REQUEST *request, int modreturn, int depth,
 				return FALSE;
 			}
 
-			DEBUG2("%.*s Evaluating %s(%.*s) -> %s",
+			RDEBUG2("%.*s Evaluating %s(%.*s) -> %s",
 			       depth, filler,
 			       invert ? "!" : "", p - start, start,
 			       (result != FALSE) ? "TRUE" : "FALSE");
 
-			DEBUG4(">>> GOT result %d", result);
+			RDEBUG4(">>> GOT result %d", result);
 
 			/*
 			 *	Not evaluating it.  We may be just
 			 *	parsing it.
 			 */
 		} else if (request) {
-			DEBUG2("%.*s Skipping %s(%.*s)",
+			RDEBUG2("%.*s Skipping %s(%.*s)",
 			       depth, filler,
 			       invert ? "!" : "", p - start, start);
 		}
 
 		if (invert) {
-			DEBUG4(">>> INVERTING result");
+			RDEBUG4(">>> INVERTING result");
 			result = (result == FALSE);
 			invert = FALSE;
 		}
@@ -770,7 +802,7 @@ int radius_evaluate_condition(REQUEST *request, int modreturn, int depth,
 		/*
 		 *	Don't evaluate it.
 		 */
-		DEBUG4(">>> EVALUATE %d ::%s::",
+		RDEBUG4(">>> EVALUATE %d ::%s::",
 			evaluate_next_condition, p);
 
 		while ((*p == ' ') || (*p == '\t')) p++;
@@ -782,18 +814,40 @@ int radius_evaluate_condition(REQUEST *request, int modreturn, int depth,
 		    ((*p == '!') && (p[1] != '=') && (p[1] != '~')) ||
 		    ((p[0] == '&') && (p[1] == '&')) ||
 		    ((p[0] == '|') && (p[1] == '|'))) {
-			DEBUG4(">>> AT EOL2a");
+			RDEBUG4(">>> AT EOL2a");
 			*ptr = p;
 			if (evaluate_next_condition) *presult = result;
 			return TRUE;
 		}
 	} /* loop over the input condition */
 
-	DEBUG4(">>> AT EOL2b");
+	RDEBUG4(">>> AT EOL2b");
 	*ptr = p;
 	if (evaluate_next_condition) *presult = result;
 	return TRUE;
 }
+#endif
+
+static void fix_up(REQUEST *request)
+{
+	VALUE_PAIR *vp;
+
+	request->username = NULL;
+	request->password = NULL;
+	
+	for (vp = request->packet->vps; vp != NULL; vp = vp->next) {
+		if ((vp->attribute == PW_USER_NAME) &&
+		    !request->username) {
+			request->username = vp;
+			
+		} else if (vp->attribute == PW_STRIPPED_USER_NAME) {
+			request->username = vp;
+			
+		} else if (vp->attribute == PW_USER_PASSWORD) {
+			request->password = vp;
+		}
+	}
+}
 
 /*
  *	The pairmove() function in src/lib/valuepair.c does all sorts of
@@ -803,7 +857,7 @@ int radius_evaluate_condition(REQUEST *request, int modreturn, int depth,
  *	only paircopy() those attributes that we're really going to
  *	use.
  */
-static void my_pairmove(REQUEST *request, VALUE_PAIR **to, VALUE_PAIR *from)
+void radius_pairmove(REQUEST *request, VALUE_PAIR **to, VALUE_PAIR *from)
 {
 	int i, j, count, from_count, to_count, tailto;
 	VALUE_PAIR *vp, *next, **last;
@@ -859,7 +913,7 @@ static void my_pairmove(REQUEST *request, VALUE_PAIR **to, VALUE_PAIR *from)
 	edited = rad_malloc(sizeof(*edited) * to_count);
 	memset(edited, 0, sizeof(*edited) * to_count);
 
-	DEBUG4("::: FROM %d TO %d MAX %d", from_count, to_count, count);
+	RDEBUG4("::: FROM %d TO %d MAX %d", from_count, to_count, count);
 
 	/*
 	 *	Now that we have the lists initialized, start working
@@ -868,7 +922,7 @@ static void my_pairmove(REQUEST *request, VALUE_PAIR **to, VALUE_PAIR *from)
 	for (i = 0; i < from_count; i++) {
 		int found;
 
-		DEBUG4("::: Examining %s", from_list[i]->name);
+		RDEBUG4("::: Examining %s", from_list[i]->name);
 
 		/*
 		 *	Attribute should be appended, OR the "to" list
@@ -901,7 +955,7 @@ static void my_pairmove(REQUEST *request, VALUE_PAIR **to, VALUE_PAIR *from)
 			 *	the one in the "from" list.
 			 */
 			if (from_list[i]->operator == T_OP_SET) {
-				DEBUG4("::: OVERWRITING %s FROM %d TO %d",
+				RDEBUG4("::: OVERWRITING %s FROM %d TO %d",
 				       to_list[j]->name, i, j);
 				pairfree(&to_list[j]);
 				to_list[j] = from_list[i];
@@ -958,7 +1012,7 @@ static void my_pairmove(REQUEST *request, VALUE_PAIR **to, VALUE_PAIR *from)
 				case T_OP_SUB:
 					if (rcode == 0) {
 					delete:
-						DEBUG4("::: DELETING %s FROM %d TO %d",
+						RDEBUG4("::: DELETING %s FROM %d TO %d",
 						       from_list[i]->name, i, j);
 						pairfree(&to_list[j]);
 						to_list[j] = NULL;
@@ -971,7 +1025,7 @@ static void my_pairmove(REQUEST *request, VALUE_PAIR **to, VALUE_PAIR *from)
 					 */
 				case T_OP_LE:
 					if (rcode > 0) {
-						DEBUG4("::: REPLACING %s FROM %d TO %d",
+						RDEBUG4("::: REPLACING %s FROM %d TO %d",
 						       from_list[i]->name, i, j);
 						pairfree(&to_list[j]);
 						to_list[j] = from_list[i];
@@ -982,7 +1036,7 @@ static void my_pairmove(REQUEST *request, VALUE_PAIR **to, VALUE_PAIR *from)
 
 				case T_OP_GE:
 					if (rcode < 0) {
-						DEBUG4("::: REPLACING %s FROM %d TO %d",
+						RDEBUG4("::: REPLACING %s FROM %d TO %d",
 						       from_list[i]->name, i, j);
 						pairfree(&to_list[j]);
 						to_list[j] = from_list[i];
@@ -1010,7 +1064,7 @@ static void my_pairmove(REQUEST *request, VALUE_PAIR **to, VALUE_PAIR *from)
 			    (from_list[i]->operator == T_OP_GE) ||
 			    (from_list[i]->operator == T_OP_SET)) {
 			append:
-				DEBUG4("::: APPENDING %s FROM %d TO %d",
+				RDEBUG4("::: APPENDING %s FROM %d TO %d",
 				       from_list[i]->name, i, tailto);
 				to_list[tailto++] = from_list[i];
 				from_list[i] = NULL;
@@ -1028,17 +1082,18 @@ static void my_pairmove(REQUEST *request, VALUE_PAIR **to, VALUE_PAIR *from)
 	}
 	free(from_list);
 
-	DEBUG4("::: TO in %d out %d", to_count, tailto);
+	RDEBUG4("::: TO in %d out %d", to_count, tailto);
 
 	/*
 	 *	Re-chain the "to" list.
 	 */
 	*to = NULL;
 	last = to;
+
 	for (i = 0; i < tailto; i++) {
 		if (!to_list[i]) continue;
 		
-		DEBUG4("::: to[%d] = %s", i, to_list[i]->name);
+		RDEBUG4("::: to[%d] = %s", i, to_list[i]->name);
 
 		/*
 		 *	Mash the operator to a simple '='.  The
@@ -1051,24 +1106,23 @@ static void my_pairmove(REQUEST *request, VALUE_PAIR **to, VALUE_PAIR *from)
 
 		*last = to_list[i];
 		last = &(*last)->next;
-
-		/*
-		 *	Fix dumb cache issues
-		 */
-		if ((i >= to_count) || edited[i]) {
-			if (to_list[i]->attribute == PW_USER_NAME) {
-				request->username = to_list[i];
-				
-			} else if (to_list[i]->attribute == PW_USER_PASSWORD) {
-				request->password = to_list[i];
-			}
-		}
 	}
+
+	/*
+	 *	Fix dumb cache issues
+	 */
+	if (to == &request->packet->vps) {
+		fix_up(request);
+	} else if (request->parent && (to == &request->parent->packet->vps)) {
+		fix_up(request->parent);
+	}
+
 	free(to_list);
 	free(edited);
 }
 
 
+#ifdef WITH_UNLANG
 /*
  *     Copied shamelessly from conffile.c, to simplify the API for
  *     now...
@@ -1125,11 +1179,13 @@ int radius_update_attrlist(REQUEST *request, CONF_SECTION *cs,
 	} else if (strcmp(name, "reply") == 0) {
 		output_vps = &request_vps->reply->vps;
 
+#ifdef WITH_PROXY
 	} else if (strcmp(name, "proxy-request") == 0) {
 		if (request->proxy) output_vps = &request_vps->proxy->vps;
 
 	} else if (strcmp(name, "proxy-reply") == 0) {
 		if (request->proxy_reply) output_vps = &request->proxy_reply->vps;
+#endif
 
 	} else if (strcmp(name, "config") == 0) {
 		output_vps = &request_vps->config_items;
@@ -1145,7 +1201,7 @@ int radius_update_attrlist(REQUEST *request, CONF_SECTION *cs,
 
 	newlist = paircopy(input_vps);
 	if (!newlist) {
-		DEBUG2("Out of memory");
+		RDEBUG2("Out of memory");
 		return RLM_MODULE_FAIL;
 	}
 
@@ -1181,8 +1237,8 @@ int radius_update_attrlist(REQUEST *request, CONF_SECTION *cs,
 			}
 
 			if (!pairparsevalue(vp, value)) {
-				DEBUG2("ERROR: Failed parsing value \"%s\" for attribute %s: %s",
-				       value, vp->name, librad_errstr);
+				RDEBUG2("ERROR: Failed parsing value \"%s\" for attribute %s: %s",
+				       value, vp->name, fr_strerror());
 				pairfree(&newlist);
 				return RLM_MODULE_FAIL;
 			}
@@ -1191,7 +1247,8 @@ int radius_update_attrlist(REQUEST *request, CONF_SECTION *cs,
 		vp = vp->next;
 	}
 
-	my_pairmove(request, output_vps, newlist);
+	radius_pairmove(request, output_vps, newlist);
 
 	return RLM_MODULE_UPDATED;
 }
+#endif
