@@ -1,7 +1,7 @@
 /*
  * listen.c	Handle socket stuff
  *
- * Version:	$Id: listen.c,v 1.125 2008/06/06 12:56:22 aland Exp $
+ * Version:	$Id$
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,10 +22,9 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: listen.c,v 1.125 2008/06/06 12:56:22 aland Exp $")
+RCSID("$Id$")
 
 #include <freeradius-devel/radiusd.h>
-#include <freeradius-devel/radius_snmp.h>
 #include <freeradius-devel/modules.h>
 #include <freeradius-devel/rad_assert.h>
 #include <freeradius-devel/vqp.h>
@@ -69,24 +68,47 @@ typedef struct listen_socket_t {
 	 */
 	fr_ipaddr_t	ipaddr;
 	int		port;
+#ifdef SO_BINDTODEVICE
+	const char		*interface;
+#endif
 	RADCLIENT_LIST	*clients;
 } listen_socket_t;
 
+static rad_listen_t *listen_alloc(RAD_LISTEN_TYPE type);
+
 /*
  *	Find a per-socket client.
  */
 RADCLIENT *client_listener_find(const rad_listen_t *listener,
-				       const fr_ipaddr_t *ipaddr)
+				const fr_ipaddr_t *ipaddr, int src_port)
 {
-	const RADCLIENT_LIST *clients;
+#ifdef WITH_DYNAMIC_CLIENTS
+	int rcode;
+	listen_socket_t *sock;
+	REQUEST *request;
+	RADCLIENT *created;
+#endif
+	time_t now;
+	RADCLIENT *client;
+	RADCLIENT_LIST *clients;
 
 	rad_assert(listener != NULL);
 	rad_assert(ipaddr != NULL);
 
-	rad_assert((listener->type == RAD_LISTEN_AUTH) ||
-		   (listener->type == RAD_LISTEN_ACCT) ||
-		   (listener->type == RAD_LISTEN_VQP) ||
-		   (listener->type == RAD_LISTEN_DHCP));
+	rad_assert((listener->type == RAD_LISTEN_AUTH)
+#ifdef WITH_STATS
+		   || (listener->type == RAD_LISTEN_NONE)
+#endif
+#ifdef WITH_ACCOUNTING
+		   || (listener->type == RAD_LISTEN_ACCT)
+#endif
+#ifdef WITH_VMPS
+		   || (listener->type == RAD_LISTEN_VQP)
+#endif
+#ifdef WITH_DHCP
+		   || (listener->type == RAD_LISTEN_DHCP)
+#endif
+		   );
 
 	clients = ((listen_socket_t *)listener->data)->clients;
 
@@ -95,7 +117,180 @@ RADCLIENT *client_listener_find(const rad_listen_t *listener,
 	 */
 	rad_assert(clients != NULL);
 
-	return client_find(clients, ipaddr);
+	client = client_find(clients, ipaddr);
+	if (!client) {
+		static time_t last_printed = 0;
+		char name[256], buffer[128];
+					
+#ifdef WITH_DYNAMIC_CLIENTS
+	unknown:		/* used only for dynamic clients */
+#endif
+
+		/*
+		 *	DoS attack quenching, but only in debug mode.
+		 *	If they're running in debug mode, show them
+		 *	every packet.
+		 */
+		if (debug_flag == 0) {
+			now = time(NULL);
+			if (last_printed == now) return NULL;
+			
+			last_printed = now;
+		}
+
+		listener->print(listener, name, sizeof(name));
+
+		radlog(L_ERR, "Ignoring request to %s from unknown client %s port %d",
+		       name, inet_ntop(ipaddr->af, &ipaddr->ipaddr,
+				       buffer, sizeof(buffer)),
+		       src_port);
+		return NULL;
+	}
+
+#ifndef WITH_DYNAMIC_CLIENTS
+	return client;		/* return the found client. */
+#else
+
+	/*
+	 *	No server defined, and it's not dynamic.  Return it.
+	 */
+	if (!client->client_server && !client->dynamic) return client;
+
+	now = time(NULL);
+	
+	/*
+	 *	It's a dynamically generated client, check it.
+	 */
+	if (client->dynamic) {
+		/*
+		 *	Lives forever.  Return it.
+		 */
+		if (client->lifetime == 0) return client;
+		
+		/*
+		 *	Rate-limit the deletion of known clients.
+		 *	This makes them last a little longer, but
+		 *	prevents the server from melting down if (say)
+		 *	10k clients all expire at once.
+		 */
+		if (now == client->last_new_client) return client;
+
+		/*
+		 *	It's not dead yet.  Return it.
+		 */
+		if ((client->created + client->lifetime) > now) return client;
+		
+		/*
+		 *	It's dead, Jim.  Delete it.
+		 */
+		client_delete(clients, client);
+
+		/*
+		 *	Go find the enclosing network again.
+		 */
+		client = client_find(clients, ipaddr);
+
+		/*
+		 *	WTF?
+		 */
+		if (!client) goto unknown;
+		if (!client->client_server) goto unknown;
+
+		/*
+		 *	At this point, 'client' is the enclosing
+		 *	network that configures where dynamic clients
+		 *	can be defined.
+		 */
+		rad_assert(client->dynamic == 0);
+	} else {
+		/*
+		 *	The IP is unknown, so we've found an enclosing
+		 *	network.  Enable DoS protection.  We only
+		 *	allow one new client per second.  Known
+		 *	clients aren't subject to this restriction.
+		 */
+		if (now == client->last_new_client) goto unknown;
+	}
+
+	client->last_new_client = now;
+
+	request = request_alloc();
+	if (!request) goto unknown;
+
+	request->listener = listener;
+	request->client = client;
+	request->packet = rad_alloc(0);
+	if (!request->packet) {
+		request_free(&request);
+		goto unknown;
+	}
+	request->reply = rad_alloc(0);
+	if (!request->reply) {
+		request_free(&request);
+		goto unknown;
+	}
+	request->packet->timestamp = request->timestamp;
+	request->number = 0;
+	request->priority = listener->type;
+	request->server = client->client_server;
+	request->root = &mainconfig;
+
+	/*
+	 *	Run a fake request through the given virtual server.
+	 *	Look for FreeRADIUS-Client-IP-Address
+	 *	         FreeRADIUS-Client-Secret
+	 *		...
+	 *
+	 *	and create the RADCLIENT structure from that.
+	 */
+
+	sock = listener->data;
+	request->packet->sockfd = listener->fd;
+	request->packet->src_ipaddr = *ipaddr;
+	request->packet->src_port = 0; /* who cares... */
+	request->packet->dst_ipaddr = sock->ipaddr;
+	request->packet->dst_port = sock->port;
+
+	request->reply->sockfd = request->packet->sockfd;
+	request->reply->dst_ipaddr = request->packet->src_ipaddr;
+	request->reply->src_ipaddr = request->packet->dst_ipaddr;
+	request->reply->dst_port = request->packet->src_port;
+	request->reply->src_port = request->packet->dst_port;
+	request->reply->id = request->packet->id;
+	request->reply->code = 0; /* UNKNOWN code */
+
+	
+	DEBUG("server %s {", request->server);
+
+	rcode = module_authorize(0, request);
+
+	DEBUG("} # server %s", request->server);
+
+	if (rcode != RLM_MODULE_OK) {
+		request_free(&request);
+		goto unknown;
+	}
+
+	/*
+	 *	If the client was updated by rlm_dynamic_clients,
+	 *	don't create the client from attribute-value pairs.
+	 */
+	if (request->client == client) {
+		created = client_create(clients, request);
+	} else {
+		created = request->client;
+
+		/*
+		 *	This frees the client if it isn't valid.
+		 */
+		if (!client_validate(clients, client, created)) goto unknown;
+	}
+	request_free(&request);
+
+	if (!created) goto unknown;
+
+	return created;
+#endif
 }
 
 static int listen_bind(rad_listen_t *this);
@@ -112,6 +307,9 @@ static int rad_status_server(REQUEST *request)
 	DICT_VALUE *dval;
 
 	switch (request->listener->type) {
+#ifdef WITH_STATS
+	case RAD_LISTEN_NONE:
+#endif
 	case RAD_LISTEN_AUTH:
 		dval = dict_valbyname(PW_AUTZ_TYPE, "Status-Server");
 		if (dval) {
@@ -164,24 +362,33 @@ static int rad_status_server(REQUEST *request)
 		return 0;
 	}
 
+#ifdef WITH_STATS
+	/*
+	 *	Full statistics are available only on a statistics
+	 *	socket.
+	 */
+	if (request->listener->type == RAD_LISTEN_NONE) {
+		request_stats_reply(request);
+	}
+#endif
+
 	return 0;
 }
 
 
 static int socket_print(rad_listen_t *this, char *buffer, size_t bufsize)
 {
+	size_t len;
 	listen_socket_t *sock = this->data;
 	const char *name;
-	char ip_buf[256];
-
-	if ((sock->ipaddr.af == AF_INET) &&
-	    (sock->ipaddr.ipaddr.ip4addr.s_addr == htonl(INADDR_ANY))) {
-		strcpy(ip_buf, "*");
-	} else {
-		ip_ntoh(&sock->ipaddr, ip_buf, sizeof(ip_buf));
-	}
 
 	switch (this->type) {
+#ifdef WITH_STATS
+	case RAD_LISTEN_NONE:	/* what a hack... */
+		name = "status";
+		break;
+#endif
+
 	case RAD_LISTEN_AUTH:
 		name = "authentication";
 		break;
@@ -215,13 +422,41 @@ static int socket_print(rad_listen_t *this, char *buffer, size_t bufsize)
 		break;
 	}
 
-	if (!this->server) {
-		return snprintf(buffer, bufsize, "%s address %s port %d",
-				name, ip_buf, sock->port);
+#define FORWARD len = strlen(buffer); if (len >= (bufsize + 1)) return 0;buffer += len;bufsize -= len
+#define ADDSTRING(_x) strlcpy(buffer, _x, bufsize);FORWARD
+
+	ADDSTRING(name);
+
+#ifdef SO_BINDTODEVICE
+	if (sock->interface) {
+		ADDSTRING(" interface ");
+		ADDSTRING(sock->interface);
+	}
+#endif
+
+	ADDSTRING(" address ");
+	
+	if ((sock->ipaddr.af == AF_INET) &&
+	    (sock->ipaddr.ipaddr.ip4addr.s_addr == htonl(INADDR_ANY))) {
+		strlcpy(buffer, "*", bufsize);
+	} else {
+		ip_ntoh(&sock->ipaddr, buffer, bufsize);
+	}
+	FORWARD;
+
+	ADDSTRING(" port ");
+	snprintf(buffer, bufsize, "%d", sock->port);
+	FORWARD;
+
+	if (this->server) {
+		ADDSTRING(" as server ");
+		ADDSTRING(this->server);
 	}
 
-	return snprintf(buffer, bufsize, "%s address %s port %d as server %s",
-			name, ip_buf, sock->port, this->server);
+#undef ADDSTRING
+#undef FORWARD
+
+	return 1;
 }
 
 
@@ -265,7 +500,7 @@ static int common_socket_parse(CONF_SECTION *cs, rad_listen_t *this)
 			      &listen_port, "0");
 	if (rcode < 0) return -1;
 
-	if ((listen_port < 0) || (listen_port > 65500)) {
+	if ((listen_port < 0) || (listen_port > 65535)) {
 			cf_log_err(cf_sectiontoitem(cs),
 				   "Invalid value for \"port\"");
 			return -1;
@@ -274,18 +509,6 @@ static int common_socket_parse(CONF_SECTION *cs, rad_listen_t *this)
 	sock->ipaddr = ipaddr;
 	sock->port = listen_port;
 
-	/*
-	 *	And bind it to the port.
-	 */
-	if (listen_bind(this) < 0) {
-		char buffer[128];
-		cf_log_err(cf_sectiontoitem(cs),
-			   "Error binding to port for %s port %d",
-			   ip_ntoh(&sock->ipaddr, buffer, sizeof(buffer)),
-			   sock->port);
-		return -1;
-	}
-
 	/*
 	 *	If we can bind to interfaces, do so,
 	 *	else don't.
@@ -298,7 +521,6 @@ static int common_socket_parse(CONF_SECTION *cs, rad_listen_t *this)
 #else
 		const char *value;
 		CONF_PAIR *cp = cf_pair_find(cs, "interface");
-		struct ifreq ifreq;
 
 		rad_assert(cp != NULL);
 		value = cf_pair_value(cp);
@@ -307,19 +529,22 @@ static int common_socket_parse(CONF_SECTION *cs, rad_listen_t *this)
 				   "No interface name given");
 			return -1;
 		}
-
-		strcpy(ifreq.ifr_name, value);
-
-		if (setsockopt(this->fd, SOL_SOCKET, SO_BINDTODEVICE,
-			       (char *)&ifreq, sizeof(ifreq)) < 0) {
-			cf_log_err(cf_sectiontoitem(cs),
-				   "Failed binding to interface %s: %s",
-				   value, strerror(errno));
-			return -1;
-		} /* else it worked. */
+		sock->interface = value;
 #endif
 	}
 
+	/*
+	 *	And bind it to the port.
+	 */
+	if (listen_bind(this) < 0) {
+		char buffer[128];
+		cf_log_err(cf_sectiontoitem(cs),
+			   "Error binding to port for %s port %d",
+			   ip_ntoh(&sock->ipaddr, buffer, sizeof(buffer)),
+			   sock->port);
+		return -1;
+	}
+
 #ifdef WITH_PROXY
 	/*
 	 *	Proxy sockets don't have clients.
@@ -380,7 +605,7 @@ static int common_socket_parse(CONF_SECTION *cs, rad_listen_t *this)
 	sock->clients = clients_parse_section(client_cs);
 	if (!sock->clients) {
 		cf_log_err(cf_sectiontoitem(cs),
-			   "Failed to find any clients for this listen section");
+			   "Failed to load clients for this listen section");
 		return -1;
 	}
 
@@ -443,6 +668,74 @@ static int proxy_socket_send(rad_listen_t *listener, REQUEST *request)
 }
 #endif
 
+#ifdef WITH_STATS
+/*
+ *	Check if an incoming request is "ok"
+ *
+ *	It takes packets, not requests.  It sees if the packet looks
+ *	OK.  If so, it does a number of sanity checks on it.
+  */
+static int stats_socket_recv(rad_listen_t *listener,
+			    RAD_REQUEST_FUNP *pfun, REQUEST **prequest)
+{
+	ssize_t		rcode;
+	int		code, src_port;
+	RADIUS_PACKET	*packet;
+	RADCLIENT	*client;
+	fr_ipaddr_t	src_ipaddr;
+
+	rcode = rad_recv_header(listener->fd, &src_ipaddr, &src_port, &code);
+	if (rcode < 0) return 0;
+
+	RAD_STATS_TYPE_INC(listener, total_requests);
+
+	if (rcode < 20) {	/* AUTH_HDR_LEN */
+		RAD_STATS_TYPE_INC(listener, total_malformed_requests);
+		return 0;
+	}
+
+	if ((client = client_listener_find(listener,
+					   &src_ipaddr, src_port)) == NULL) {
+		rad_recv_discard(listener->fd);
+		RAD_STATS_TYPE_INC(listener, total_invalid_requests);
+		return 0;
+	}
+
+	/*
+	 *	We only understand Status-Server on this socket.
+	 */
+	if (code != PW_STATUS_SERVER) {
+		DEBUG("Ignoring packet code %d sent to Status-Server port",
+		      code);
+		rad_recv_discard(listener->fd);
+		RAD_STATS_TYPE_INC(listener, total_unknown_types);
+		RAD_STATS_CLIENT_INC(listener, client, total_unknown_types);
+		return 0;
+	}
+
+	/*
+	 *	Now that we've sanity checked everything, receive the
+	 *	packet.
+	 */
+	packet = rad_recv(listener->fd, 1); /* require message authenticator */
+	if (!packet) {
+		RAD_STATS_TYPE_INC(listener, total_malformed_requests);
+		DEBUG("%s", fr_strerror());
+		return 0;
+	}
+
+	if (!received_request(listener, packet, prequest, client)) {
+		RAD_STATS_TYPE_INC(listener, total_packets_dropped);
+		RAD_STATS_CLIENT_INC(listener, client, total_packets_dropped);
+		rad_free(&packet);
+		return 0;
+	}
+
+	*pfun = rad_status_server;
+	return 1;
+}
+#endif
+
 
 /*
  *	Check if an incoming request is "ok"
@@ -457,40 +750,23 @@ static int auth_socket_recv(rad_listen_t *listener,
 	int		code, src_port;
 	RADIUS_PACKET	*packet;
 	RAD_REQUEST_FUNP fun = NULL;
-	char		buffer[128];
 	RADCLIENT	*client;
 	fr_ipaddr_t	src_ipaddr;
 
 	rcode = rad_recv_header(listener->fd, &src_ipaddr, &src_port, &code);
 	if (rcode < 0) return 0;
 
-	RAD_SNMP_TYPE_INC(listener, total_requests);
+	RAD_STATS_TYPE_INC(listener, total_requests);
 
 	if (rcode < 20) {	/* AUTH_HDR_LEN */
-		RAD_SNMP_TYPE_INC(listener, total_malformed_requests);
+		RAD_STATS_TYPE_INC(listener, total_malformed_requests);
 		return 0;
 	}
 
 	if ((client = client_listener_find(listener,
-					   &src_ipaddr)) == NULL) {
+					   &src_ipaddr, src_port)) == NULL) {
 		rad_recv_discard(listener->fd);
-		RAD_SNMP_TYPE_INC(listener, total_invalid_requests);
-
-		if (debug_flag > 0) {
-			char name[1024];
-
-			listener->print(listener, name, sizeof(name));
-
-			/*
-			 *	This is debugging rather than logging, so that
-			 *	DoS attacks don't affect us.
-			 */
-			DEBUG("Ignoring request to %s from unknown client %s port %d",
-			      name,
-			      inet_ntop(src_ipaddr.af, &src_ipaddr.ipaddr,
-					buffer, sizeof(buffer)), src_port);
-		}
-
+		RAD_STATS_TYPE_INC(listener, total_invalid_requests);
 		return 0;
 	}
 
@@ -499,15 +775,15 @@ static int auth_socket_recv(rad_listen_t *listener,
 	 */
 	switch(code) {
 	case PW_AUTHENTICATION_REQUEST:
-		RAD_SNMP_CLIENT_INC(listener, client, requests);
+		RAD_STATS_CLIENT_INC(listener, client, total_requests);
 		fun = rad_authenticate;
 		break;
 
 	case PW_STATUS_SERVER:
 		if (!mainconfig.status_server) {
 			rad_recv_discard(listener->fd);
-			RAD_SNMP_TYPE_INC(listener, total_packets_dropped);
-			RAD_SNMP_CLIENT_INC(listener, client, packets_dropped);
+			RAD_STATS_TYPE_INC(listener, total_packets_dropped);
+			RAD_STATS_CLIENT_INC(listener, client, total_packets_dropped);
 			DEBUG("WARNING: Ignoring Status-Server request due to security configuration");
 			return 0;
 		}
@@ -516,8 +792,8 @@ static int auth_socket_recv(rad_listen_t *listener,
 
 	default:
 		rad_recv_discard(listener->fd);
-		RAD_SNMP_INC(rad_snmp.auth.total_unknown_types);
-		RAD_SNMP_CLIENT_INC(listener, client, unknown_types);
+		RAD_STATS_INC(radius_auth_stats.total_unknown_types);
+		RAD_STATS_CLIENT_INC(listener, client, total_unknown_types);
 
 		DEBUG("Invalid packet code %d sent to authentication port from client %s port %d : IGNORED",
 		      code, client->shortname, src_port);
@@ -531,14 +807,14 @@ static int auth_socket_recv(rad_listen_t *listener,
 	 */
 	packet = rad_recv(listener->fd, client->message_authenticator);
 	if (!packet) {
-		RAD_SNMP_TYPE_INC(listener, total_malformed_requests);
-		DEBUG("%s", librad_errstr);
+		RAD_STATS_TYPE_INC(listener, total_malformed_requests);
+		DEBUG("%s", fr_strerror());
 		return 0;
 	}
 
 	if (!received_request(listener, packet, prequest, client)) {
-		RAD_SNMP_TYPE_INC(listener, total_packets_dropped);
-		RAD_SNMP_CLIENT_INC(listener, client, packets_dropped);
+		RAD_STATS_TYPE_INC(listener, total_packets_dropped);
+		RAD_STATS_CLIENT_INC(listener, client, total_packets_dropped);
 		rad_free(&packet);
 		return 0;
 	}
@@ -559,40 +835,23 @@ static int acct_socket_recv(rad_listen_t *listener,
 	int		code, src_port;
 	RADIUS_PACKET	*packet;
 	RAD_REQUEST_FUNP fun = NULL;
-	char		buffer[128];
 	RADCLIENT	*client;
 	fr_ipaddr_t	src_ipaddr;
 
 	rcode = rad_recv_header(listener->fd, &src_ipaddr, &src_port, &code);
 	if (rcode < 0) return 0;
 
-	RAD_SNMP_TYPE_INC(listener, total_requests);
+	RAD_STATS_TYPE_INC(listener, total_requests);
 
 	if (rcode < 20) {	/* AUTH_HDR_LEN */
-		RAD_SNMP_TYPE_INC(listener, total_malformed_requests);
+		RAD_STATS_TYPE_INC(listener, total_malformed_requests);
 		return 0;
 	}
 
 	if ((client = client_listener_find(listener,
-					   &src_ipaddr)) == NULL) {
+					   &src_ipaddr, src_port)) == NULL) {
 		rad_recv_discard(listener->fd);
-		RAD_SNMP_TYPE_INC(listener, total_invalid_requests);
-
-		/*
-		 *	This is debugging rather than logging, so that
-		 *	DoS attacks don't affect us.
-		 */
-		if (debug_flag > 0) {
-			char name[1024];
-
-			listener->print(listener, name, sizeof(name));
-
-			DEBUG("Ignoring request to %s from unknown client %s port %d",
-			      name,
-			      inet_ntop(src_ipaddr.af, &src_ipaddr.ipaddr,
-					buffer, sizeof(buffer)), src_port);
-		}
-
+		RAD_STATS_TYPE_INC(listener, total_invalid_requests);
 		return 0;
 	}
 
@@ -601,15 +860,15 @@ static int acct_socket_recv(rad_listen_t *listener,
 	 */
 	switch(code) {
 	case PW_ACCOUNTING_REQUEST:
-		RAD_SNMP_CLIENT_INC(listener, client, requests);
+		RAD_STATS_CLIENT_INC(listener, client, total_requests);
 		fun = rad_accounting;
 		break;
 
 	case PW_STATUS_SERVER:
 		if (!mainconfig.status_server) {
 			rad_recv_discard(listener->fd);
-			RAD_SNMP_TYPE_INC(listener, total_packets_dropped);
-			RAD_SNMP_CLIENT_INC(listener, client, unknown_types);
+			RAD_STATS_TYPE_INC(listener, total_packets_dropped);
+			RAD_STATS_CLIENT_INC(listener, client, total_unknown_types);
 
 			DEBUG("WARNING: Ignoring Status-Server request due to security configuration");
 			return 0;
@@ -619,8 +878,8 @@ static int acct_socket_recv(rad_listen_t *listener,
 
 	default:
 		rad_recv_discard(listener->fd);
-		RAD_SNMP_TYPE_INC(listener, total_unknown_types);
-		RAD_SNMP_CLIENT_INC(listener, client, unknown_types);
+		RAD_STATS_TYPE_INC(listener, total_unknown_types);
+		RAD_STATS_CLIENT_INC(listener, client, total_unknown_types);
 
 		DEBUG("Invalid packet code %d sent to a accounting port from client %s port %d : IGNORED",
 		      code, client->shortname, src_port);
@@ -633,8 +892,8 @@ static int acct_socket_recv(rad_listen_t *listener,
 	 */
 	packet = rad_recv(listener->fd, 0);
 	if (!packet) {
-		RAD_SNMP_TYPE_INC(listener, total_malformed_requests);
-		radlog(L_ERR, "%s", librad_errstr);
+		RAD_STATS_TYPE_INC(listener, total_malformed_requests);
+		radlog(L_ERR, "%s", fr_strerror());
 		return 0;
 	}
 
@@ -642,8 +901,8 @@ static int acct_socket_recv(rad_listen_t *listener,
 	 *	There can be no duplicate accounting packets.
 	 */
 	if (!received_request(listener, packet, prequest, client)) {
-		RAD_SNMP_TYPE_INC(listener, total_packets_dropped);
-		RAD_SNMP_CLIENT_INC(listener, client, packets_dropped);
+		RAD_STATS_TYPE_INC(listener, total_packets_dropped);
+		RAD_STATS_CLIENT_INC(listener, client, total_packets_dropped);
 		rad_free(&packet);
 		return 0;
 	}
@@ -667,7 +926,7 @@ static int proxy_socket_recv(rad_listen_t *listener,
 
 	packet = rad_recv(listener->fd, 0);
 	if (!packet) {
-		radlog(L_ERR, "%s", librad_errstr);
+		radlog(L_ERR, "%s", fr_strerror());
 		return 0;
 	}
 
@@ -760,53 +1019,27 @@ static int proxy_socket_decode(UNUSED rad_listen_t *listener, REQUEST *request)
 }
 #endif
 
-#ifdef WITH_SNMP
-static int radius_snmp_recv(rad_listen_t *listener,
-			    UNUSED RAD_REQUEST_FUNP *pfun,
-			    UNUSED REQUEST **prequest)
-{
-	if ((rad_snmp.smux_fd >= 0) &&
-	    (rad_snmp.smux_event == SMUX_READ)) {
-		smux_read();
-	}
-
-	/*
-	 *  If we've got to re-connect, then do so now,
-	 *  before calling select again.
-	 */
-	if (rad_snmp.smux_event == SMUX_CONNECT) {
-		smux_connect();
-	}
-
-	/*
-	 *	Reset this every time, as the smux connect may have
-	 *	opened a new socket.
-	 */
-	listener->fd = rad_snmp.smux_fd;
-
-	return 0;
-}
-
-
-static int radius_snmp_print(UNUSED rad_listen_t *this, char *buffer, size_t bufsize)
-{
-	return snprintf(buffer, bufsize, "SMUX with OID .1.3.6.1.4.1.11344.1.1.1");
-}
-
-#endif
-
 #include "dhcpd.c"
 
+#include "command.c"
+
 static const rad_listen_master_t master_listen[RAD_LISTEN_MAX] = {
+#ifdef WITH_STATS
+	{ common_socket_parse, NULL,
+	  stats_socket_recv, auth_socket_send,
+	  socket_print, client_socket_encode, client_socket_decode },
+#else
+	/*
+	 *	This always gets defined.
+	 */
 	{ NULL, NULL, NULL, NULL, NULL, NULL, NULL},	/* RAD_LISTEN_NONE */
+#endif
 
 #ifdef WITH_PROXY
 	/* proxying */
 	{ common_socket_parse, NULL,
 	  proxy_socket_recv, proxy_socket_send,
 	  socket_print, proxy_socket_encode, proxy_socket_decode },
-#else
-	{ NULL, NULL, NULL, NULL, NULL, NULL, NULL},
 #endif
 
 	/* authentication */
@@ -819,8 +1052,6 @@ static const rad_listen_master_t master_listen[RAD_LISTEN_MAX] = {
 	{ common_socket_parse, NULL,
 	  acct_socket_recv, acct_socket_send,
 	  socket_print, client_socket_encode, client_socket_decode},
-#else
-	{ NULL, NULL, NULL, NULL, NULL, NULL, NULL},
 #endif
 
 #ifdef WITH_DETAIL
@@ -828,8 +1059,6 @@ static const rad_listen_master_t master_listen[RAD_LISTEN_MAX] = {
 	{ detail_parse, detail_free,
 	  detail_recv, detail_send,
 	  detail_print, detail_encode, detail_decode },
-#else
-	{ NULL, NULL, NULL, NULL, NULL, NULL, NULL},
 #endif
 
 #ifdef WITH_VMPS
@@ -837,8 +1066,6 @@ static const rad_listen_master_t master_listen[RAD_LISTEN_MAX] = {
 	{ common_socket_parse, NULL,
 	  vqp_socket_recv, vqp_socket_send,
 	  socket_print, vqp_socket_encode, vqp_socket_decode },
-#else
-	{ NULL, NULL, NULL, NULL, NULL, NULL, NULL},
 #endif
 
 #ifdef WITH_DHCP
@@ -846,11 +1073,15 @@ static const rad_listen_master_t master_listen[RAD_LISTEN_MAX] = {
 	{ dhcp_socket_parse, NULL,
 	  dhcp_socket_recv, dhcp_socket_send,
 	  socket_print, dhcp_socket_encode, dhcp_socket_decode },
-#else
-	{ NULL, NULL, NULL, NULL, NULL, NULL, NULL},
 #endif
 
-	{ NULL, NULL, NULL, NULL, NULL, NULL, NULL}	/* RAD_LISTEN_SNMP */
+#ifdef WITH_COMMAND_SOCKET
+	/* TCP command socket */
+	{ command_socket_parse, NULL,
+	  command_domain_accept, command_domain_send,
+	  command_socket_print, command_socket_encode, command_socket_decode },
+#endif
+
 };
 
 
@@ -860,6 +1091,8 @@ static const rad_listen_master_t master_listen[RAD_LISTEN_MAX] = {
  */
 static int listen_bind(rad_listen_t *this)
 {
+	struct sockaddr_storage salocal;
+	socklen_t	salen;
 	listen_socket_t *sock = this->data;
 
 	/*
@@ -908,13 +1141,78 @@ static int listen_bind(rad_listen_t *this)
 		}
 	}
 
-	this->fd = fr_socket(&sock->ipaddr, sock->port);
+	/*
+	 *	Copy fr_socket() here, as we may need to bind to a device.
+	 */
+	this->fd = socket(sock->ipaddr.af, SOCK_DGRAM, 0);
 	if (this->fd < 0) {
-		radlog(L_ERR, "ERROR: Failed to open socket: %s",
-		       librad_errstr);
+		radlog(L_ERR, "Failed opening socket: %s", strerror(errno));
 		return -1;
 	}
+		
+#ifdef SO_BINDTODEVICE
+	/*
+	 *	Bind to a device BEFORE touching IP addresses.
+	 */
+	if (sock->interface) {
+		struct ifreq ifreq;
+		strcpy(ifreq.ifr_name, sock->interface);
+		
+		if (setsockopt(this->fd, SOL_SOCKET, SO_BINDTODEVICE,
+			       (char *)&ifreq, sizeof(ifreq)) < 0) {
+			close(this->fd);
+			radlog(L_ERR, "Failed opening to interface %s: %s",
+			       sock->interface, strerror(errno));
+			return -1;
+		} /* else it worked. */
+	}
+#endif
 
+#ifdef WITH_UDPFROMTO
+	/*
+	 *	Initialize udpfromto for all sockets.
+	 */
+	if (udpfromto_init(this->fd) != 0) {
+		close(this->fd);
+		return -1;
+	}
+#endif
+	
+	/*
+	 *	Set up sockaddr stuff.
+	 */
+	if (!fr_ipaddr2sockaddr(&sock->ipaddr, sock->port, &salocal, &salen)) {
+		close(this->fd);
+		return -1;
+	}
+		
+#ifdef HAVE_STRUCT_SOCKADDR_IN6
+	if (sock->ipaddr.af == AF_INET6) {
+		/*
+		 *	Listening on '::' does NOT get you IPv4 to
+		 *	IPv6 mapping.  You've got to listen on an IPv4
+		 *	address, too.  This makes the rest of the server
+		 *	design a little simpler.
+		 */
+#ifdef IPV6_V6ONLY
+		
+		if (IN6_IS_ADDR_UNSPECIFIED(&sock->ipaddr.ipaddr.ip6addr)) {
+			int on = 1;
+			
+			setsockopt(this->fd, IPPROTO_IPV6, IPV6_V6ONLY,
+				   (char *)&on, sizeof(on));
+		}
+#endif /* IPV6_V6ONLY */
+	}
+#endif /* HAVE_STRUCT_SOCKADDR_IN6 */
+		
+	if (bind(this->fd, (struct sockaddr *) &salocal, salen) < 0) {
+		close(this->fd);
+		radlog(L_ERR, "Failed binding to socket: %s\n",
+		       strerror(errno));
+		return -1;
+	}
+	
 	/*
 	 *	FreeBSD jail issues.  We bind to 0.0.0.0, but the
 	 *	kernel instead binds us to a 1.2.3.4.  If this
@@ -932,22 +1230,8 @@ static int listen_bind(rad_listen_t *this)
 			return -1;
 		}
 
-		if (src.ss_family == AF_INET) {
-			struct sockaddr_in	s4;
-			
-			memcpy(&s4, &src, sizeof(s4));
-			sock->ipaddr.ipaddr.ip4addr = s4.sin_addr;
-			sock->port = ntohs(s4.sin_port);
-			
-#ifdef HAVE_STRUCT_SOCKADDR_IN6
-		} else if (src.ss_family == AF_INET6) {
-			struct sockaddr_in6	s6;
-			
-			memcpy(&s6, &src, sizeof(s6));
-			sock->ipaddr.ipaddr.ip6addr = s6.sin6_addr;
-			sock->port = ntohs(s6.sin6_port);
-#endif
-		} else {
+		if (!fr_sockaddr2ipaddr(&src, sizeof_src,
+					&sock->ipaddr, &sock->port)) {
 			radlog(L_ERR, "Socket has unsupported address family");
 			return -1;
 		}
@@ -994,11 +1278,22 @@ static rad_listen_t *listen_alloc(RAD_LISTEN_TYPE type)
 	this->decode = master_listen[this->type].decode;
 
 	switch (type) {
+#ifdef WITH_STATS
+	case RAD_LISTEN_NONE:
+#endif
 	case RAD_LISTEN_AUTH:
+#ifdef WITH_ACCOUNTING
 	case RAD_LISTEN_ACCT:
+#endif
+#ifdef WITH_PROXY
 	case RAD_LISTEN_PROXY:
+#endif
+#ifdef WITH_VMPS
 	case RAD_LISTEN_VQP:
+#endif
+#ifdef WITH_DHCP
 	case RAD_LISTEN_DHCP:
+#endif
 		this->data = rad_malloc(sizeof(listen_socket_t));
 		memset(this->data, 0, sizeof(listen_socket_t));
 		break;
@@ -1009,6 +1304,13 @@ static rad_listen_t *listen_alloc(RAD_LISTEN_TYPE type)
 		break;
 #endif
 
+#ifdef WITH_COMMAND_SOCKET
+	case RAD_LISTEN_COMMAND:
+		this->data = rad_malloc(sizeof(fr_command_socket_t));
+		memset(this->data, 0, sizeof(fr_command_socket_t));
+		break;
+#endif
+
 	default:
 		rad_assert("Unsupported option!" == NULL);
 		break;
@@ -1090,6 +1392,9 @@ rad_listen_t *proxy_new_listener()
 #endif
 
 static const FR_NAME_NUMBER listen_compare[] = {
+#ifdef WITH_STATS
+	{ "status",	RAD_LISTEN_NONE },
+#endif
 	{ "auth",	RAD_LISTEN_AUTH },
 #ifdef WITH_ACCOUNTING
 	{ "acct",	RAD_LISTEN_ACCT },
@@ -1105,8 +1410,9 @@ static const FR_NAME_NUMBER listen_compare[] = {
 #endif
 #ifdef WITH_DHCP
 	{ "dhcp",	RAD_LISTEN_DHCP },
-#else
-	{ "dhcp",	RAD_LISTEN_NONE },
+#endif
+#ifdef WITH_COMMAND_SOCKET
+	{ "control",	RAD_LISTEN_COMMAND },
 #endif
 	{ NULL, 0 },
 };
@@ -1132,9 +1438,8 @@ static rad_listen_t *listen_parse(CONF_SECTION *cs, const char *server)
 		return NULL;
 	}
 
-	type = fr_str2int(listen_compare, listen_type,
-			    RAD_LISTEN_NONE);
-	if (type == RAD_LISTEN_NONE) {
+	type = fr_str2int(listen_compare, listen_type, -1);
+	if (type < 0) {
 		cf_log_err(cf_sectiontoitem(cs),
 			   "Invalid type \"%s\" in listen section.",
 			   listen_type);
@@ -1185,7 +1490,7 @@ int listen_init(CONF_SECTION *config, rad_listen_t **head)
 {
 	int		override = FALSE;
 	int		rcode;
-	CONF_SECTION	*cs;
+	CONF_SECTION	*cs = NULL;
 	rad_listen_t	**last;
 	rad_listen_t	*this;
 	fr_ipaddr_t	server_ipaddr;
@@ -1254,13 +1559,14 @@ int listen_init(CONF_SECTION *config, rad_listen_t **head)
 		if (!sock->clients) {
 			cf_log_err(cf_sectiontoitem(config),
 				   "Failed to find any clients for this listen section");
+			listen_free(&this);
 			return -1;
 		}
 
 		if (listen_bind(this) < 0) {
-			listen_free(&this);
 			listen_free(head);
 			radlog(L_ERR, "There appears to be another RADIUS server running on the authentication port %d", sock->port);
+			listen_free(&this);
 			return -1;
 		}
 		auth_port = sock->port;	/* may have been updated in listen_bind */
@@ -1432,7 +1738,7 @@ int listen_init(CONF_SECTION *config, rad_listen_t **head)
 		 */
 		if (!*head) return -1;
 
-		if (defined_proxy) goto do_snmp;
+		if (defined_proxy) goto done;
 
 		/*
 		 *	Find the first authentication port,
@@ -1496,23 +1802,7 @@ int listen_init(CONF_SECTION *config, rad_listen_t **head)
 		}
 	}
 
- do_snmp:			/* used only in proxy code. */
-#endif
-
-#ifdef WITH_SNMP
-	if (radius_snmp_init(config)) {
-		this = rad_malloc(sizeof(*this));
-		memset(this, 0, sizeof(*this));
-
-		this->type = RAD_LISTEN_SNMP;
-		this->fd = rad_snmp.smux_fd;
-
-		this->recv = radius_snmp_recv;
-		this->print = radius_snmp_print;
-
-		*last = this;
-		last = &(this->next);
-	}
+ done:			/* used only in proxy code. */
 #endif
 
 	return 0;
@@ -1547,3 +1837,48 @@ void listen_free(rad_listen_t **head)
 
 	*head = NULL;
 }
+
+#ifdef WITH_STATS
+RADCLIENT_LIST *listener_find_client_list(const fr_ipaddr_t *ipaddr,
+					  int port)
+{
+	rad_listen_t *this;
+
+	for (this = mainconfig.listen; this != NULL; this = this->next) {
+		listen_socket_t *sock;
+
+		if ((this->type != RAD_LISTEN_AUTH) &&
+		    (this->type != RAD_LISTEN_ACCT)) continue;
+		
+		sock = this->data;
+
+		if ((sock->port == port) &&
+		    (fr_ipaddr_cmp(ipaddr, &sock->ipaddr) == 0)) {
+			return sock->clients;
+		}
+	}
+
+	return NULL;
+}
+
+rad_listen_t *listener_find_byipaddr(const fr_ipaddr_t *ipaddr, int port)
+{
+	rad_listen_t *this;
+
+	for (this = mainconfig.listen; this != NULL; this = this->next) {
+		listen_socket_t *sock;
+
+		if ((this->type != RAD_LISTEN_AUTH) &&
+		    (this->type != RAD_LISTEN_ACCT)) continue;
+		
+		sock = this->data;
+
+		if ((sock->port == port) &&
+		    (fr_ipaddr_cmp(ipaddr, &sock->ipaddr) == 0)) {
+			return this;
+		}
+	}
+
+	return NULL;
+}
+#endif
