@@ -1,7 +1,7 @@
 /*
  * mainconf.c	Handle the server's configuration.
  *
- * Version:	$Id: mainconfig.c,v 1.152 2008/06/05 09:48:22 aland Exp $
+ * Version:	$Id$
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: mainconfig.c,v 1.152 2008/06/05 09:48:22 aland Exp $")
+RCSID("$Id$")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/modules.h>
@@ -59,15 +59,20 @@ RCSID("$Id: mainconfig.c,v 1.152 2008/06/05 09:48:22 aland Exp $")
 #endif
 
 struct main_config_t mainconfig;
+char *request_log_file = NULL;
+char *debug_log_file = NULL;
+char *debug_condition = NULL;
 
 /*
  *	Temporary local variables for parsing the configuration
  *	file.
  */
-static uid_t server_uid;
+#ifndef __MINGW32__
+uid_t server_uid;
 static gid_t server_gid;
 static const char *uid_name = NULL;
 static const char *gid_name = NULL;
+#endif
 static const char *chroot_dir = NULL;
 static int allow_core_dumps = 0;
 static const char *radlog_dest = NULL;
@@ -143,7 +148,7 @@ static const FR_NAME_NUMBER str2fac[] = {
  *  Security configuration for the server.
  */
 static const CONF_PARSER security_config[] = {
-	{ "max_attributes",  PW_TYPE_INTEGER, 0, &librad_max_attributes, Stringify(0) },
+	{ "max_attributes",  PW_TYPE_INTEGER, 0, &fr_max_attributes, Stringify(0) },
 	{ "reject_delay",  PW_TYPE_INTEGER, 0, &mainconfig.reject_delay, Stringify(0) },
 	{ "status_server", PW_TYPE_BOOLEAN, 0, &mainconfig.status_server, "no"},
 	{ NULL, -1, 0, NULL, NULL }
@@ -158,6 +163,8 @@ static const CONF_PARSER logdest_config[] = {
 	{ "syslog_facility",  PW_TYPE_STRING_PTR, 0, &syslog_facility, Stringify(0) },
 
 	{ "file", PW_TYPE_STRING_PTR, -1, &mainconfig.log_file, "${logdir}/radius.log" },
+	{ "requests", PW_TYPE_STRING_PTR, -1, &request_log_file, NULL },
+	{ "debug_file", PW_TYPE_STRING_PTR, -1, &debug_log_file, NULL },
 	{ NULL, -1, 0, NULL, NULL }
 };
 
@@ -197,7 +204,7 @@ static const CONF_PARSER server_config[] = {
 	{ "logdir",             PW_TYPE_STRING_PTR, 0, &radlog_dir,        "${localstatedir}/log"},
 	{ "libdir",             PW_TYPE_STRING_PTR, 0, &radlib_dir,        "${prefix}/lib"},
 	{ "radacctdir",         PW_TYPE_STRING_PTR, 0, &radacct_dir,       "${logdir}/radacct" },
-	{ "hostname_lookups",   PW_TYPE_BOOLEAN,    0, &librad_dodns,      "no" },
+	{ "hostname_lookups",   PW_TYPE_BOOLEAN,    0, &fr_dns_lookups,      "no" },
 	{ "max_request_time", PW_TYPE_INTEGER, 0, &mainconfig.max_request_time, Stringify(MAX_REQUEST_TIME) },
 	{ "cleanup_delay", PW_TYPE_INTEGER, 0, &mainconfig.cleanup_delay, Stringify(CLEANUP_DELAY) },
 	{ "max_requests", PW_TYPE_INTEGER, 0, &mainconfig.max_requests, Stringify(MAX_REQUESTS) },
@@ -229,6 +236,8 @@ static const CONF_PARSER server_config[] = {
 	{ "log_auth_goodpass", PW_TYPE_BOOLEAN, 0, &mainconfig.log_auth_goodpass, NULL },
 	{ "log_stripped_names", PW_TYPE_BOOLEAN, 0, &log_stripped_names, NULL },
 
+	{  "security", PW_TYPE_SUBSECTION, 0, NULL, (const void *) security_config },
+
 	{ NULL, -1, 0, NULL, NULL }
 };
 
@@ -394,6 +403,9 @@ static int r_mkdir(const char *part)
 }
 
 
+#ifndef __MINGW32__
+int did_setuid = FALSE;
+
 /*
  *  Do chroot, if requested.
  *
@@ -401,7 +413,6 @@ static int r_mkdir(const char *part)
  */
 static int switch_users(CONF_SECTION *cs)
 {
-	int did_setuid = FALSE;
 	CONF_PAIR *cp;
 
 #ifdef HAVE_SYS_RESOURCE_H
@@ -499,10 +510,26 @@ static int switch_users(CONF_SECTION *cs)
 #endif
 
 #ifdef HAVE_PWD_H
-	if (uid_name && (setuid(server_uid) < 0)) {
-		fprintf(stderr, "%s: Failed setting user to %s: %s",
-			progname, uid_name, strerror(errno));
-		return 0;
+	if (uid_name) {
+
+#ifndef HAVE_SETRESUID
+/*
+ *	Fake out setresuid with something that's close.
+ */
+#define setresuid(_a, _b, _c) setuid(_b)
+#endif
+
+		if (setresuid(-1, server_uid, geteuid()) < 0) {
+			fprintf(stderr, "%s: Failed switching uid: %s\n",
+				progname, strerror(errno));
+			return 0;
+		}
+		
+		if (geteuid() != server_uid) {
+			fprintf(stderr, "%s: Failed switching uid: UID is incorrect\n",
+				progname);
+			return 0;
+		}
 	}
 
 	/*
@@ -605,6 +632,7 @@ static int switch_users(CONF_SECTION *cs)
 
 	return 1;
 }
+#endif
 
 
 static const FR_NAME_NUMBER str2dest[] = {
@@ -713,10 +741,12 @@ int read_mainconfig(int reload)
 		}
 	}
 
+#ifndef __MINGW32__
 	/*
 	 *	We should really switch users earlier in the process.
 	 */
 	if (!switch_users(cs)) exit(1);
+#endif
 
 	/* Initialize the dictionary */
 	cp = cf_pair_find(cs, "dictionary");
@@ -725,7 +755,7 @@ int read_mainconfig(int reload)
 	DEBUG2("including dictionary file %s/%s", p, RADIUS_DICTIONARY);
 	if (dict_init(p, RADIUS_DICTIONARY) != 0) {
 		radlog(L_ERR, "Errors reading dictionary: %s",
-				librad_errstr);
+				fr_strerror());
 		return -1;
 	}
 
@@ -745,7 +775,9 @@ int read_mainconfig(int reload)
 	cf_section_free(&mainconfig.config);
 	mainconfig.config = cs;
 
-	clients_parse_section(cs);
+	if (!clients_parse_section(cs)) {
+		return -1;
+	}
 
 	DEBUG2("%s: #### Loading Realms and Home Servers ####", mainconfig.name);
 
@@ -777,7 +809,7 @@ int read_mainconfig(int reload)
 		 */
 		debug_flag = mainconfig.debug_level;
 	}
-	librad_debug = debug_flag;
+	fr_debug_flag = debug_flag;
 	old_debug_level = mainconfig.debug_level;
 
 	/*
