@@ -1,7 +1,7 @@
 /*
  * modcall.c
  *
- * Version:	$Id: modcall.c,v 1.100 2008/06/05 10:32:54 aland Exp $
+ * Version:	$Id$
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: modcall.c,v 1.100 2008/06/05 10:32:54 aland Exp $")
+RCSID("$Id$")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/modpriv.h>
@@ -52,7 +52,7 @@ struct modcallable {
 #ifdef WITH_UNLANG
 	       MOD_IF, MOD_ELSE, MOD_ELSIF, MOD_UPDATE, MOD_SWITCH, MOD_CASE,
 #endif
-	       MOD_POLICY, MOD_REFERENCE } type;
+	       MOD_POLICY, MOD_REFERENCE, MOD_XLAT } type;
 	int method;
 	int actions[RLM_MODULE_NUMCODES];
 };
@@ -81,6 +81,12 @@ typedef struct {
 	CONF_SECTION *ref_cs;
 } modref;
 
+typedef struct {
+	modcallable mc;
+	int exec;
+	char *xlat_name;
+} modxlat;
+
 static const FR_NAME_NUMBER grouptype_table[] = {
 	{ "", GROUPTYPE_SIMPLE },
 	{ "redundant ", GROUPTYPE_REDUNDANT },
@@ -120,6 +126,16 @@ static modcallable *mod_reftocallable(modref *p)
 	return (modcallable *)p;
 }
 
+static modxlat *mod_callabletoxlat(modcallable *p)
+{
+	rad_assert(p->type==MOD_XLAT);
+	return (modxlat *)p;
+}
+static modcallable *mod_xlattocallable(modxlat *p)
+{
+	return (modcallable *)p;
+}
+
 /* modgroups are grown by adding a modcallable to the end */
 /* FIXME: This is O(N^2) */
 static void add_child(modgroup *g, modcallable *c)
@@ -256,7 +272,7 @@ static int call_modsingle(int component, modsingle *sp, REQUEST *request,
 {
 	int myresult = default_result;
 
-	DEBUG3("  modsingle[%s]: calling %s (%s) for request %d",
+	RDEBUG3("  modsingle[%s]: calling %s (%s) for request %d",
 	       comp2str[component], sp->modinst->name,
 	       sp->modinst->entry->name, request->number);
 	safe_lock(sp->modinst);
@@ -269,9 +285,9 @@ static int call_modsingle(int component, modsingle *sp, REQUEST *request,
 	myresult = sp->modinst->entry->module->methods[component](
 			sp->modinst->insthandle, request);
 
-	request->module = "<server-core>";
+	request->module = "";
 	safe_unlock(sp->modinst);
-	DEBUG3("  modsingle[%s]: returned from %s (%s) for request %d",
+	RDEBUG3("  modsingle[%s]: returned from %s (%s) for request %d",
 	       comp2str[component], sp->modinst->name,
 	       sp->modinst->entry->name, request->number);
 
@@ -372,14 +388,14 @@ int modcall(int component, modcallable *c, REQUEST *request)
 			myresult = stack.result[stack.pointer];
 
 			if (!was_if) { /* error */
-				DEBUG2("%.*s ... skipping %s for request %d: No preceding \"if\"",
+				RDEBUG2("%.*s ... skipping %s for request %d: No preceding \"if\"",
 				       stack.pointer + 1, modcall_spaces,
 				       group_name[child->type],
 				       request->number);
 				goto unroll;
 			}
 			if (if_taken) {
-				DEBUG2("%.*s ... skipping %s for request %d: Preceding \"if\" was taken",
+				RDEBUG2("%.*s ... skipping %s for request %d: Preceding \"if\" was taken",
 				       stack.pointer + 1, modcall_spaces,
 				       group_name[child->type],
 				       request->number);
@@ -394,14 +410,14 @@ int modcall(int component, modcallable *c, REQUEST *request)
 			int condition = TRUE;
 			const char *p = child->name;
 
-			DEBUG2("%.*s? %s %s",
+			RDEBUG2("%.*s? %s %s",
 			       stack.pointer + 1, modcall_spaces,
 			       (child->type == MOD_IF) ? "if" : "elsif",
 			       child->name);
 
 			if (radius_evaluate_condition(request, myresult,
 						      0, &p, TRUE, &condition)) {
-				DEBUG2("%.*s? %s %s -> %s",
+				RDEBUG2("%.*s? %s %s -> %s",
 				       stack.pointer + 1, modcall_spaces,
 				       (child->type == MOD_IF) ? "if" : "elsif",
 				       child->name, (condition != FALSE) ? "TRUE" : "FALSE");
@@ -441,19 +457,37 @@ int modcall(int component, modcallable *c, REQUEST *request)
 			const char *server = request->server;
 
 			if (server == mr->ref_name) {
-				DEBUG("WARNING: Suppressing recursive call to server %s", server);
+				RDEBUG("WARNING: Suppressing recursive call to server %s", server);
 				myresult = RLM_MODULE_NOOP;
 				goto handle_result;
 			}
 			
 			request->server = mr->ref_name;
-			DEBUG("server %s { # nested call", mr->ref_name);
+			RDEBUG("server %s { # nested call", mr->ref_name);
 			myresult = indexed_modcall(component, 0, request);
-			DEBUG("} # server %s with nested call", mr->ref_name);
+			RDEBUG("} # server %s with nested call", mr->ref_name);
 			request->server = server;
 			goto handle_result;
 		}
 
+		if (child->type == MOD_XLAT) {
+			modxlat *mx = mod_callabletoxlat(child);
+			char buffer[128];
+
+			if (!mx->exec) {
+				radius_xlat(buffer, sizeof(buffer),
+					    mx->xlat_name, request, NULL);
+			} else {
+				RDEBUG("`%s`", mx->xlat_name);
+				radius_exec_program(mx->xlat_name, request,
+						    0, NULL, 0,
+						    request->packet->vps,
+						    NULL, 1);
+			}
+					    
+			goto handle_result;
+		}
+
 		/*
 		 *	Child is a group that has children of it's own.
 		 */
@@ -547,7 +581,7 @@ int modcall(int component, modcallable *c, REQUEST *request)
 #endif
 
 			default:
-				DEBUG2("Internal sanity check failed in modcall %d", child->type);
+				RDEBUG2("Internal sanity check failed in modcall %d", child->type);
 				exit(1); /* internal sanity check failure */
 				break;
 			}
@@ -555,7 +589,7 @@ int modcall(int component, modcallable *c, REQUEST *request)
 
 			stack.start[stack.pointer] = stack.children[stack.pointer];
 
-			DEBUG2("%.*s- entering %s %s",
+			RDEBUG2("%.*s- entering %s %s {...}",
 			       stack.pointer, modcall_spaces,
 			       group_name[child->type],
 			       child->name ? child->name : "");
@@ -567,7 +601,7 @@ int modcall(int component, modcallable *c, REQUEST *request)
 				/*
 				 *	Print message for NULL group
 				 */
-				DEBUG2("%.*s- %s %s returns %s",
+				RDEBUG2("%.*s- %s %s returns %s",
 				       stack.pointer + 1, modcall_spaces,
 				       group_name[child->type],
 				       child->name ? child->name : "",
@@ -594,7 +628,7 @@ int modcall(int component, modcallable *c, REQUEST *request)
 		myresult = call_modsingle(child->method, sp, request,
 					  default_component_results[component]);
 	handle_result:
-		DEBUG2("%.*s[%s] returns %s",
+		RDEBUG2("%.*s[%s] returns %s",
 		       stack.pointer + 1, modcall_spaces,
 		       child->name ? child->name : "",
 		       fr_int2str(rcode_table, myresult, "??"));
@@ -708,7 +742,7 @@ int modcall(int component, modcallable *c, REQUEST *request)
 				}
 				break;
 			default:
-				DEBUG2("Internal sanity check failed in modcall  next %d", child->type);
+				RDEBUG2("Internal sanity check failed in modcall  next %d", child->type);
 				exit(1);
 		}
 
@@ -725,7 +759,7 @@ int modcall(int component, modcallable *c, REQUEST *request)
 
 			if (stack.pointer == 0) break;
 
-			DEBUG2("%.*s- %s %s returns %s",
+			RDEBUG2("%.*s- %s %s returns %s",
 			       stack.pointer + 1, modcall_spaces,
 			       group_name[parent->type],
 			       parent->name ? parent->name : "",
@@ -775,29 +809,29 @@ static void dump_mc(modcallable *c, int indent)
 
 	if(c->type==MOD_SINGLE) {
 		modsingle *single = mod_callabletosingle(c);
-		DEBUG("%.*s%s {", indent, "\t\t\t\t\t\t\t\t\t\t\t",
+		RDEBUG("%.*s%s {", indent, "\t\t\t\t\t\t\t\t\t\t\t",
 			single->modinst->name);
 	} else {
 		modgroup *g = mod_callabletogroup(c);
 		modcallable *p;
-		DEBUG("%.*s%s {", indent, "\t\t\t\t\t\t\t\t\t\t\t",
+		RDEBUG("%.*s%s {", indent, "\t\t\t\t\t\t\t\t\t\t\t",
 		      group_name[c->type]);
 		for(p = g->children;p;p = p->next)
 			dump_mc(p, indent+1);
 	}
 
 	for(i = 0; i<RLM_MODULE_NUMCODES; ++i) {
-		DEBUG("%.*s%s = %s", indent+1, "\t\t\t\t\t\t\t\t\t\t\t",
+		RDEBUG("%.*s%s = %s", indent+1, "\t\t\t\t\t\t\t\t\t\t\t",
 		      fr_int2str(rcode_table, i, "??"),
 		      action2str(c->actions[i]));
 	}
 
-	DEBUG("%.*s}", indent, "\t\t\t\t\t\t\t\t\t\t\t");
+	RDEBUG("%.*s}", indent, "\t\t\t\t\t\t\t\t\t\t\t");
 }
 
 static void dump_tree(int comp, modcallable *c)
 {
-	DEBUG("[%s]", comp2str[comp]);
+	RDEBUG("[%s]", comp2str[comp]);
 	dump_mc(c, 0);
 }
 #else
@@ -1190,7 +1224,7 @@ static modcallable *do_compile_modupdate(modcallable *parent,
 		vp = cf_pairtovp(cp);
 		if (!vp) {
 			pairfree(&head);
-			cf_log_err(ci, "ERROR: %s", librad_errstr);
+			cf_log_err(ci, "ERROR: %s", fr_strerror());
 			return NULL;
 		}
 
@@ -1355,6 +1389,38 @@ static modcallable *do_compile_modserver(modcallable *parent,
 	return csingle;
 }
 
+static modcallable *do_compile_modxlat(modcallable *parent,
+				       int component, const char *fmt)
+{
+	modcallable *csingle;
+	modxlat *mx;
+
+	mx = rad_malloc(sizeof(*mx));
+	memset(mx, 0, sizeof(*mx));
+
+	csingle = mod_xlattocallable(mx);
+	csingle->parent = parent;
+	csingle->next = NULL;
+	csingle->name = "expand";
+	csingle->type = MOD_XLAT;
+	csingle->method = component;
+
+	memcpy(csingle->actions, defaultactions[component][GROUPTYPE_SIMPLE],
+	       sizeof(csingle->actions));
+	
+	mx->xlat_name = strdup(fmt);
+	if (fmt[0] != '%') {
+		char *p;
+		mx->exec = TRUE;
+
+		strcpy(mx->xlat_name, fmt + 1);
+		p = strrchr(mx->xlat_name, '`');
+		if (p) *p = '\0';
+	}
+
+	return csingle;
+}
+
 /*
  *	redundant, etc. can refer to modules or groups, but not much else.
  */
@@ -1629,12 +1695,20 @@ static modcallable *do_compile_modsingle(modcallable *parent,
 			return NULL;
 		}
 
+		if (((modrefname[0] == '%') && (modrefname[1] == '{')) ||
+		    (modrefname[0] == '`')) {
+			return do_compile_modxlat(parent, component,
+						  modrefname);
+		}
+
 		/*
 		 *	See if the module is a virtual one.  If so,
 		 *	return that, rather than doing anything here.
 		 */
 		subcs = NULL;
 		cs = cf_section_find("instantiate");
+		if (cs) subcs = cf_section_sub_find_name2(cs, NULL,
+							  modrefname);
 		if (!subcs) {
 			cs = cf_section_find("policy");
 			if (cs) subcs = cf_section_sub_find_name2(cs, NULL,
@@ -1673,7 +1747,7 @@ static modcallable *do_compile_modsingle(modcallable *parent,
 	this = NULL;
 
 	if (modules && cf_section_sub_find_name2(modules, NULL, modrefname)) {
-		this = find_module_instance(modules, modrefname);
+		this = find_module_instance(modules, modrefname, 1);
 	}
 
 	if (!this) do {
@@ -1694,7 +1768,7 @@ static modcallable *do_compile_modsingle(modcallable *parent,
 				buffer[p - modrefname] = '\0';
 				component = i;
 				
-				this = find_module_instance(cf_section_find("modules"), buffer);
+				this = find_module_instance(cf_section_find("modules"), buffer, 1);
 				if (this &&
 				    !this->entry->module->methods[i]) {
 					*modname = NULL;
