@@ -1,7 +1,7 @@
 /*
  * realms.c	Realm handling code
  *
- * Version:     $Id: realms.c,v 1.44 2008/02/10 15:23:33 aland Exp $
+ * Version:     $Id$
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: realms.c,v 1.44 2008/02/10 15:23:33 aland Exp $")
+RCSID("$Id$")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/rad_assert.h>
@@ -32,12 +32,38 @@ RCSID("$Id: realms.c,v 1.44 2008/02/10 15:23:33 aland Exp $")
 #include <ctype.h>
 #include <fcntl.h>
 
+#ifdef HAVE_REGEX_H
+#include <regex.h>
+
+/*
+ *  For POSIX Regular expressions.
+ *  (0) Means no extended regular expressions.
+ *  REG_EXTENDED means use extended regular expressions.
+ */
+#ifndef REG_EXTENDED
+#define REG_EXTENDED (0)
+#endif
+
+#ifndef REG_NOSUB
+#define REG_NOSUB (0)
+#endif
+
+#ifndef REG_ICASE
+#define REG_ICASE (0)
+#endif
+#endif
+
 static rbtree_t *realms_byname = NULL;
 
-static rbtree_t	*home_servers_byaddr = NULL;
-static rbtree_t	*home_servers_byname = NULL;
+#ifdef HAVE_REGEX_H
+typedef struct realm_regex_t {
+	REALM	*realm;
+	struct realm_regex_t *next;
+} realm_regex_t;
 
-static rbtree_t	*home_pools_byname = NULL;
+static realm_regex_t *realms_regex = NULL;
+
+#endif /* HAVE_REGEX_H */
 
 typedef struct realm_config_t {
 	CONF_SECTION	*cs;
@@ -50,6 +76,16 @@ typedef struct realm_config_t {
 
 static realm_config_t *realm_config = NULL;
 
+#ifdef WITH_PROXY
+static rbtree_t	*home_servers_byaddr = NULL;
+static rbtree_t	*home_servers_byname = NULL;
+#ifdef WITH_STATS
+static int home_server_max_number = 0;
+static rbtree_t	*home_servers_bynumber = NULL;
+#endif
+
+static rbtree_t	*home_pools_byname = NULL;
+
 /*
  *  Map the proxy server configuration parameters to variables.
  */
@@ -76,6 +112,7 @@ static const CONF_PARSER proxy_config[] = {
 
 	{ NULL, -1, 0, NULL, NULL }
 };
+#endif
 
 static int realm_name_cmp(const void *one, const void *two)
 {
@@ -86,6 +123,7 @@ static int realm_name_cmp(const void *one, const void *two)
 }
 
 
+#ifdef WITH_PROXY
 static int home_server_name_cmp(const void *one, const void *two)
 {
 	const home_server *a = one;
@@ -102,12 +140,29 @@ static int home_server_addr_cmp(const void *one, const void *two)
 	const home_server *a = one;
 	const home_server *b = two;
 
+	if (a->server && !b->server) return -1;
+	if (!a->server && b->server) return +1;
+	if (a->server && b->server) {
+		int rcode = a->type - b->type;
+		if (rcode != 0) return rcode;
+		return strcmp(a->server, b->server);
+	}
+
 	if (a->port < b->port) return -1;
 	if (a->port > b->port) return +1;
 
 	return fr_ipaddr_cmp(&a->ipaddr, &b->ipaddr);
 }
 
+#ifdef WITH_STATS
+static int home_server_number_cmp(const void *one, const void *two)
+{
+	const home_server *a = one;
+	const home_server *b = two;
+
+	return (a->number - b->number);
+}
+#endif
 
 static int home_pool_name_cmp(const void *one, const void *two)
 {
@@ -177,10 +232,16 @@ static size_t xlat_server_pool(UNUSED void *instance, REQUEST *request,
 
 	return strlen(out);
 }
-
+#endif
 
 void realms_free(void)
 {
+#ifdef WITH_PROXY
+#ifdef WITH_STATS
+	rbtree_free(home_servers_bynumber);
+	home_servers_bynumber = NULL;
+#endif
+
 	rbtree_free(home_servers_byname);
 	home_servers_byname = NULL;
 
@@ -189,24 +250,41 @@ void realms_free(void)
 
 	rbtree_free(home_pools_byname);
 	home_pools_byname = NULL;
+#endif
 
 	rbtree_free(realms_byname);
 	realms_byname = NULL;
 
+#ifdef HAVE_REGEX_H
+	if (realms_regex) {
+		realm_regex_t *this, *next;
+
+		for (this = realms_regex; this != NULL; this = next) {
+			next = this->next;
+			free(this->realm);
+			free(this);
+		}
+	}
+#endif
+
 	free(realm_config);
 }
 
 
+#ifdef WITH_PROXY
 static struct in_addr hs_ip4addr;
 static struct in6_addr hs_ip6addr;
 static char *hs_type = NULL;
 static char *hs_check = NULL;
+static char *hs_virtual_server = NULL;
 
 static CONF_PARSER home_server_config[] = {
 	{ "ipaddr",  PW_TYPE_IPADDR,
 	  0, &hs_ip4addr,  NULL },
 	{ "ipv6addr",  PW_TYPE_IPV6ADDR,
 	  0, &hs_ip6addr, NULL },
+	{ "virtual_server",  PW_TYPE_STRING_PTR,
+	  0, &hs_virtual_server, NULL },
 
 	{ "port", PW_TYPE_INTEGER,
 	  offsetof(home_server,port), NULL,   "0" },
@@ -227,7 +305,7 @@ static CONF_PARSER home_server_config[] = {
 	{ "status_check", PW_TYPE_STRING_PTR,
 	  0, &hs_check,   "none" },
 	{ "ping_check", PW_TYPE_STRING_PTR,
-	  0, &hs_check,   "none" },
+	  0, &hs_check,   NULL },
 
 	{ "ping_interval", PW_TYPE_INTEGER,
 	  offsetof(home_server,ping_interval), NULL,   "30" },
@@ -247,16 +325,25 @@ static CONF_PARSER home_server_config[] = {
 	{ "password",  PW_TYPE_STRING_PTR,
 	  offsetof(home_server,ping_user_password), NULL,  NULL},
 
+#ifdef WITH_STATS
+	{ "historic_average_window", PW_TYPE_INTEGER,
+	  offsetof(home_server,ema.window), NULL,  NULL },
+#endif
+
 	{ NULL, -1, 0, NULL, NULL }		/* end the list */
 
 };
 
 
-static int home_server_add(CONF_SECTION *cs, int type)
+static int home_server_add(realm_config_t *rc, CONF_SECTION *cs, int type)
 {
 	const char *name2;
 	home_server *home;
 	int dual = FALSE;
+	CONF_PAIR *cp;
+
+	free(hs_virtual_server); /* used only for printing during parsing */
+	hs_virtual_server = NULL;
 
 	name2 = cf_section_name1(cs);
 	if (!name2 || (strcasecmp(name2, "home_server") != 0)) {
@@ -282,19 +369,6 @@ static int home_server_add(CONF_SECTION *cs, int type)
 	memset(&hs_ip6addr, 0, sizeof(hs_ip6addr));
 	cf_section_parse(cs, home, home_server_config);
 
-	if (!cf_pair_find(cs, "ipaddr") &&
-	    !cf_pair_find(cs, "ipv6addr")) {
-		cf_log_err(cf_sectiontoitem(cs),
-			   "No IPv4 or IPv6 address defined for home server %s.",
-			   name2);
-		free(home);
-		free(hs_type);
-		hs_type = NULL;
-		free(hs_check);
-		hs_check = NULL;
-		return 0;
-	}
-
 	/*
 	 *	Figure out which one to use.
 	 */
@@ -306,10 +380,32 @@ static int home_server_add(CONF_SECTION *cs, int type)
 		home->ipaddr.af = AF_INET6;
 		home->ipaddr.ipaddr.ip6addr = hs_ip6addr;
 
+	} else if ((cp = cf_pair_find(cs, "virtual_server")) != NULL) {
+		home->ipaddr.af = AF_UNSPEC;
+		home->server = cf_pair_value(cp);
+		if (!home->server) {
+			cf_log_err(cf_sectiontoitem(cs),
+				   "Invalid value for virtual_server");
+			goto error;
+		}
+
+		if (!cf_section_sub_find_name2(rc->cs, "server", home->server)) {
+		  
+			cf_log_err(cf_sectiontoitem(cs),
+				   "No such server %s", home->server);
+			goto error;
+		}
+
+		free(hs_type);
+		hs_type = NULL;
+		home->secret = strdup("");
+		goto skip_port;
+
 	} else {
 		cf_log_err(cf_sectiontoitem(cs),
-			   "Internal sanity check failed for home server %s.",
+			   "No ipaddr, ipv6addr, or virtual_server defined for home server \"%s\".",
 			   name2);
+	error:
 		free(home);
 		free(hs_type);
 		hs_type = NULL;
@@ -322,29 +418,28 @@ static int home_server_add(CONF_SECTION *cs, int type)
 		cf_log_err(cf_sectiontoitem(cs),
 			   "No port, or invalid port defined for home server %s.",
 			   name2);
-		free(home);
-		free(hs_type);
-		hs_type = NULL;
-		free(hs_check);
-		hs_check = NULL;
-		return 0;
+		goto error;
 	}
 
 	if (0) {
 		cf_log_err(cf_sectiontoitem(cs),
 			   "Fatal error!  Home server %s is ourselves!",
 			   name2);
+		goto error;
+	}
+
+	if (!home->secret) {
+		cf_log_err(cf_sectiontoitem(cs),
+			   "No shared secret defined for home server %s.",
+			   name2);
 		free(home);
-		free(hs_type);
-		hs_type = NULL;
-		free(hs_check);
-		hs_check = NULL;
 		return 0;
 	}
 
 	/*
 	 *	Use a reasonable default.
 	 */
+ skip_port:
 	if (!hs_type) hs_type = strdup("auth+acct");
 
 	if (strcasecmp(hs_type, "auth") == 0) {
@@ -385,14 +480,6 @@ static int home_server_add(CONF_SECTION *cs, int type)
 	free(hs_type);
 	hs_type = NULL;
 
-	if (!home->secret) {
-		cf_log_err(cf_sectiontoitem(cs),
-			   "No shared secret defined for home server %s.",
-			   name2);
-		free(home);
-		return 0;
-	}
-
 	if (!hs_check || (strcasecmp(hs_check, "none") == 0)) {
 		home->ping_check = HOME_PING_CHECK_NONE;
 
@@ -430,28 +517,45 @@ static int home_server_add(CONF_SECTION *cs, int type)
 		}
 	}
 
-	if (rbtree_finddata(home_servers_byaddr, home)) {
+	if ((home->ipaddr.af != AF_UNSPEC) && /* could be virtual server */
+	    rbtree_finddata(home_servers_byaddr, home)) {
 		DEBUG2("Ignoring duplicate home server %s.", name2);
 		return 1;
 	}
 
 	if (!rbtree_insert(home_servers_byname, home)) {
 		cf_log_err(cf_sectiontoitem(cs),
-			   "Internal error adding home server %s.",
-			   name2);
+			   "Internal error %d adding home server %s.",
+			   __LINE__, name2);
 		free(home);
 		return 0;
 	}
 
-	if (!rbtree_insert(home_servers_byaddr, home)) {
+	if ((home->ipaddr.af != AF_UNSPEC) && /* could be virtual server */
+	    !rbtree_insert(home_servers_byaddr, home)) {
 		rbtree_deletebydata(home_servers_byname, home);
 		cf_log_err(cf_sectiontoitem(cs),
-			   "Internal error adding home server %s.",
-			   name2);
+			   "Internal error %d adding home server %s.",
+			   __LINE__, name2);
 		free(home);
 		return 0;
 	}
 
+#ifdef WITH_STATS
+	home->number = home_server_max_number++;
+	if (!rbtree_insert(home_servers_bynumber, home)) {
+		rbtree_deletebydata(home_servers_byname, home);
+		if (home->ipaddr.af != AF_UNSPEC) {
+			rbtree_deletebydata(home_servers_byname, home);
+		}
+		cf_log_err(cf_sectiontoitem(cs),
+			   "Internal error %d adding home server %s.",
+			   __LINE__, name2);
+		free(home);
+		return 0;
+	}
+#endif
+
 	if (home->response_window < 5) home->response_window = 5;
 	if (home->response_window > 60) home->response_window = 60;
 
@@ -489,20 +593,36 @@ static int home_server_add(CONF_SECTION *cs, int type)
 
 		if (!rbtree_insert(home_servers_byname, home2)) {
 			cf_log_err(cf_sectiontoitem(cs),
-				   "Internal error adding home server %s.",
-				   name2);
+				   "Internal error %d adding home server %s.",
+				   __LINE__, name2);
 			free(home2);
 			return 0;
 		}
 		
-		if (!rbtree_insert(home_servers_byaddr, home2)) {
+		if ((home->ipaddr.af != AF_UNSPEC) &&
+		    !rbtree_insert(home_servers_byaddr, home2)) {
 			rbtree_deletebydata(home_servers_byname, home2);
 			cf_log_err(cf_sectiontoitem(cs),
-				   "Internal error adding home server %s.",
-				   name2);
+				   "Internal error %d adding home server %s.",
+				   __LINE__, name2);
 			free(home2);
 			return 0;
 		}
+
+#ifdef WITH_STATS
+		home2->number = home_server_max_number++;
+		if (!rbtree_insert(home_servers_bynumber, home2)) {
+			rbtree_deletebydata(home_servers_byname, home2);
+			if (home2->ipaddr.af != AF_UNSPEC) {
+				rbtree_deletebydata(home_servers_byname, home2);
+			}
+			cf_log_err(cf_sectiontoitem(cs),
+				   "Internal error %d adding home server %s.",
+				   __LINE__, name2);
+			free(home2);
+			return 0;
+		}
+#endif
 	}
 
 	return 1;
@@ -529,6 +649,49 @@ static home_pool_t *server_pool_alloc(const char *name, home_pool_type_t type,
 	return pool;
 }
 
+static int pool_check_home_server(realm_config_t *rc, CONF_PAIR *cp,
+				  const char *name, int server_type,
+				  home_server **phome)
+{
+	home_server myhome, *home;
+	CONF_SECTION *server_cs;
+
+	if (!name) {
+		cf_log_err(cf_pairtoitem(cp),
+			   "No value given for home_server.");
+		return 0;
+	}
+
+	myhome.name = name;
+	myhome.type = server_type;
+	home = rbtree_finddata(home_servers_byname, &myhome);
+	if (home) {
+		*phome = home;
+		return 1;
+	}
+	
+	server_cs = cf_section_sub_find_name2(rc->cs, "home_server", name);
+	if (!server_cs) {
+		cf_log_err(cf_pairtoitem(cp),
+			   "Unknown home_server \"%s\".", name);
+		return 0;
+	}
+	
+	if (!home_server_add(rc, server_cs, server_type)) {
+		return 0;
+	}
+	
+	home = rbtree_finddata(home_servers_byname, &myhome);
+	if (!home) {
+		radlog(L_ERR, "Internal sanity check failed %d",
+		       __LINE__);
+		return 0;
+	}
+
+	*phome = home;
+	return 1;
+}
+
 
 static int server_pool_add(realm_config_t *rc,
 			   CONF_SECTION *cs, int server_type, int do_print)
@@ -538,6 +701,7 @@ static int server_pool_add(realm_config_t *rc,
 	const char *value;
 	CONF_PAIR *cp;
 	int num_home_servers;
+	home_server *home;
 
 	name2 = cf_section_name1(cs);
 	if (!name2 || ((strcasecmp(name2, "server_pool") != 0) &&
@@ -561,41 +725,11 @@ static int server_pool_add(realm_config_t *rc,
 	for (cp = cf_pair_find(cs, "home_server");
 	     cp != NULL;
 	     cp = cf_pair_find_next(cs, cp, "home_server")) {
-		home_server myhome, *home;
-		CONF_SECTION *server_cs;
-
 		num_home_servers++;
 
-		value = cf_pair_value(cp);
-		if (!value) {
-			cf_log_err(cf_pairtoitem(cp),
-				   "No value given for home_server.");
-			return 0;;
-		}
-
-		myhome.name = value;
-		myhome.type = server_type;
-		home = rbtree_finddata(home_servers_byname, &myhome);
-		if (home) continue;
-
-		server_cs = cf_section_sub_find_name2(rc->cs,
-						      "home_server",
-						      value);
-		if (!server_cs) {
-			cf_log_err(cf_pairtoitem(cp),
-				   "Unknown home_server \"%s\".",
-				   value);
-			return 0;
-		}
-
-		if (!home_server_add(server_cs, server_type)) {
-			return 0;
-		}
-
-		home = rbtree_finddata(home_servers_byname, &myhome);
-		if (!home) {
-			radlog(L_ERR, "Internal sanity check failed %d",
-			       __LINE__);
+		if (!pool_check_home_server(rc, cp, cf_pair_value(cp),
+					    server_type, &home)) {
+					    
 			return 0;
 		}
 	}
@@ -611,6 +745,25 @@ static int server_pool_add(realm_config_t *rc,
 				 num_home_servers);
 	pool->cs = cs;
 
+
+	/*
+	 *	Fallback servers must be defined, and must be
+	 *	virtual servers.
+	 */
+	cp = cf_pair_find(cs, "fallback");
+	if (cp) {
+		if (!pool_check_home_server(rc, cp, cf_pair_value(cp),
+					    server_type, &pool->fallback)) {
+			
+			goto error;
+		}
+
+		if (!pool->fallback->server) {
+			cf_log_err(cf_sectiontoitem(cs), "Fallback home_server %s does NOT contain a virtual_server directive.", pool->fallback->name);
+			goto error;
+		}
+	}
+
 	if (do_print) cf_log_info(cs, " home_server_pool %s {", name2);
 
 	cp = cf_pair_find(cs, "type");
@@ -646,8 +799,13 @@ static int server_pool_add(realm_config_t *rc,
 
 	cp = cf_pair_find(cs, "virtual_server");
 	if (cp) {
-		pool->virtual_server = cf_pair_value(cp);
-		if (do_print && pool->virtual_server) {
+		pool->virtual_server = cf_pair_value(cp);		
+		if (!pool->virtual_server) {
+			cf_log_err(cf_pairtoitem(cp), "No value given for virtual_server");
+			goto error;
+		}
+
+		if (do_print) {
 			cf_log_info(cs, "\tvirtual_server = %s", pool->virtual_server);
 		}
 
@@ -664,15 +822,11 @@ static int server_pool_add(realm_config_t *rc,
 	for (cp = cf_pair_find(cs, "home_server");
 	     cp != NULL;
 	     cp = cf_pair_find_next(cs, cp, "home_server")) {
-		home_server myhome, *home;
+		home_server myhome;
 
 		value = cf_pair_value(cp);
-		if (!value) {
-			cf_log_err(cf_pairtoitem(cp),
-				   "No value given for home_server.");
-			goto error;
-		}
 
+		memset(&myhome, 0, sizeof(&myhome));
 		myhome.name = value;
 		myhome.type = server_type;
 
@@ -691,6 +845,10 @@ static int server_pool_add(realm_config_t *rc,
 		pool->servers[num_home_servers++] = home;
 	} /* loop over home_server's */
 
+	if (pool->fallback && do_print) {
+		cf_log_info(cs, "\tfallback = %s", pool->fallback->name);
+	}
+
 	if (!rbtree_insert(home_pools_byname, pool)) {
 		rad_assert("Internal sanity check failed");
 		goto error;
@@ -707,18 +865,27 @@ static int server_pool_add(realm_config_t *rc,
 	free(pool);
 	return 0;
 }
-
+#endif
 
 static int old_server_add(realm_config_t *rc, CONF_SECTION *cs,
 			  const char *realm,
 			  const char *name, const char *secret,
 			  home_pool_type_t ldflag, home_pool_t **pool_p,
-			  int type)
+			  int type, const char *server)
 {
+#ifdef WITH_PROXY
 	int i, insert_point, num_home_servers;
 	home_server myhome, *home;
 	home_pool_t mypool, *pool;
 	CONF_SECTION *subcs;
+#else
+	rc = rc;		/* -Wunused */
+	realm = realm;
+	secret = secret;
+	ldflag = ldflag;
+	type = type;
+	server = server;
+#endif
 
 	/*
 	 *	LOCAL realms get sanity checked, and nothing else happens.
@@ -731,6 +898,10 @@ static int old_server_add(realm_config_t *rc, CONF_SECTION *cs,
 		return 1;
 	}
 
+#ifndef WITH_PROXY
+	return 0;		/* Not proxying.  Can't do non-LOCAL realms */
+
+#else
 	mypool.name = realm;
 	mypool.server_type = type;
 	pool = rbtree_finddata(home_pools_byname, &mypool);
@@ -750,7 +921,7 @@ static int old_server_add(realm_config_t *rc, CONF_SECTION *cs,
 	myhome.type = type;
 	home = rbtree_finddata(home_servers_byname, &myhome);
 	if (home) {
-		if (strcmp(home->secret, secret) != 0) {
+		if (secret && (strcmp(home->secret, secret) != 0)) {
 			cf_log_err(cf_sectiontoitem(cs), "Inconsistent shared secret for home server \"%s\"", name);
 			return 0;
 		}
@@ -846,13 +1017,18 @@ static int old_server_add(realm_config_t *rc, CONF_SECTION *cs,
 			p = q;
 		}
 
-		if (ip_hton(p, AF_UNSPEC, &home->ipaddr) < 0) {
-			cf_log_err(cf_sectiontoitem(cs),
-				   "Failed looking up hostname %s.",
-				   p);
-			free(home);
-			free(q);
-			return 0;
+		if (!server) {
+			if (ip_hton(p, AF_UNSPEC, &home->ipaddr) < 0) {
+				cf_log_err(cf_sectiontoitem(cs),
+					   "Failed looking up hostname %s.",
+					   p);
+				free(home);
+				free(q);
+				return 0;
+			}
+		} else {
+			home->ipaddr.af = AF_UNSPEC;
+			home->server = server;
 		}
 		free(q);
 
@@ -875,17 +1051,32 @@ static int old_server_add(realm_config_t *rc, CONF_SECTION *cs,
 		}
 
 		if (!rbtree_insert(home_servers_byname, home)) {
-			cf_log_err(cf_sectiontoitem(cs), "Internal error adding home server %s.", name);
+			cf_log_err(cf_sectiontoitem(cs), "Internal error %d adding home server %s.", __LINE__, name);
 			free(home);
 			return 0;
 		}
 
 		if (!rbtree_insert(home_servers_byaddr, home)) {
 			rbtree_deletebydata(home_servers_byname, home);
-			cf_log_err(cf_sectiontoitem(cs), "Internal error adding home server %s.", name);
+			cf_log_err(cf_sectiontoitem(cs), "Internal error %d adding home server %s.", __LINE__, name);
 			free(home);
 			return 0;
 		}
+
+#ifdef WITH_STATS
+		home->number = home_server_max_number++;
+		if (!rbtree_insert(home_servers_bynumber, home)) {
+			rbtree_deletebydata(home_servers_byname, home);
+			if (home->ipaddr.af != AF_UNSPEC) {
+				rbtree_deletebydata(home_servers_byname, home);
+			}
+			cf_log_err(cf_sectiontoitem(cs),
+				   "Internal error %d adding home server %s.",
+				   __LINE__, name);
+			free(home);
+			return 0;
+		}
+#endif
 	}
 
 	/*
@@ -933,6 +1124,7 @@ static int old_server_add(realm_config_t *rc, CONF_SECTION *cs,
 	*pool_p = pool;
 
 	return 1;
+#endif
 }
 
 static int old_realm_config(realm_config_t *rc, CONF_SECTION *cs, REALM *r)
@@ -993,7 +1185,7 @@ static int old_realm_config(realm_config_t *rc, CONF_SECTION *cs, REALM *r)
 		cf_log_info(cs, "\tauthhost = %s",  host);
 
 		if (!old_server_add(rc, cs, r->name, host, secret, ldflag,
-				    &r->auth_pool, HOME_TYPE_AUTH)) {
+				    &r->auth_pool, HOME_TYPE_AUTH, NULL)) {
 			return 0;
 		}
 	}
@@ -1027,7 +1219,27 @@ static int old_realm_config(realm_config_t *rc, CONF_SECTION *cs, REALM *r)
 		cf_log_info(cs, "\taccthost = %s", host);
 
 		if (!old_server_add(rc, cs, r->name, host, secret, ldflag,
-				    &r->acct_pool, HOME_TYPE_ACCT)) {
+				    &r->acct_pool, HOME_TYPE_ACCT, NULL)) {
+			return 0;
+		}
+	}
+
+	cp = cf_pair_find(cs, "virtual_server");
+	if (cp) {
+		host = cf_pair_value(cp);
+		if (!host) {
+			cf_log_err(cf_pairtoitem(cp), "No value specified for virtual_server");
+			return 0;
+		}
+
+		cf_log_info(cs, "\tvirtual_server = %s", host);
+
+		if (!old_server_add(rc, cs, r->name, host, "", ldflag,
+				    &r->auth_pool, HOME_TYPE_AUTH, host)) {
+			return 0;
+		}
+		if (!old_server_add(rc, cs, r->name, host, "", ldflag,
+				    &r->acct_pool, HOME_TYPE_ACCT, host)) {
 			return 0;
 		}
 	}
@@ -1039,6 +1251,7 @@ static int old_realm_config(realm_config_t *rc, CONF_SECTION *cs, REALM *r)
 }
 
 
+#ifdef WITH_PROXY
 static int add_pool_to_realm(realm_config_t *rc, CONF_SECTION *cs,
 			     const char *name, home_pool_t **dest,
 			     int server_type, int do_print)
@@ -1085,14 +1298,18 @@ static int add_pool_to_realm(realm_config_t *rc, CONF_SECTION *cs,
 
 	return 1;
 }
+#endif
+
 
 static int realm_add(realm_config_t *rc, CONF_SECTION *cs)
 {
 	const char *name2;
 	REALM *r = NULL;
 	CONF_PAIR *cp;
+#ifdef WITH_PROXY
 	home_pool_t *auth_pool, *acct_pool;
 	const char *auth_pool_name, *acct_pool_name;
+#endif
 
 	name2 = cf_section_name1(cs);
 	if (!name2 || (strcasecmp(name2, "realm") != 0)) {
@@ -1106,6 +1323,7 @@ static int realm_add(realm_config_t *rc, CONF_SECTION *cs)
 		return 0;
 	}
 
+#ifdef WITH_PROXY
 	auth_pool = acct_pool = NULL;
 	auth_pool_name = acct_pool_name = NULL;
 
@@ -1164,15 +1382,17 @@ static int realm_add(realm_config_t *rc, CONF_SECTION *cs)
 			return 0;
 		}
 	}
+#endif
 
 	cf_log_info(cs, " realm %s {", name2);
 
+#ifdef WITH_PROXY
 	/*
 	 *	The realm MAY already exist if it's an old-style realm.
 	 *	In that case, merge the old-style realm with this one.
 	 */
-	r = realm_find(name2);
-	if (r) {
+	r = realm_find2(name2);
+	if (r && (strcmp(r->name, name2) == 0)) {
 		if (cf_pair_find(cs, "auth_pool") ||
 		    cf_pair_find(cs, "acct_pool")) {
 			cf_log_err(cf_sectiontoitem(cs), "Duplicate realm \"%s\"", name2);
@@ -1186,15 +1406,34 @@ static int realm_add(realm_config_t *rc, CONF_SECTION *cs)
 		cf_log_info(cs, " } # realm %s", name2);
 		return 1;
 	}
+#endif
+
+#ifdef HAVE_REGEX_H
+	if (name2[0] == '~') {
+		regex_t reg;
+		
+		/*
+		 *	Include substring matches.
+		 */
+		if (regcomp(&reg, name2 + 1,
+			    REG_EXTENDED | REG_NOSUB | REG_ICASE) != 0) {
+			cf_log_err(cf_sectiontoitem(cs),
+				   "Invalid regex in realm \"%s\"", name2);
+			goto error;
+		}
+		regfree(&reg);
+	}
+#endif
 
 	r = rad_malloc(sizeof(*r));
 	memset(r, 0, sizeof(*r));
 
 	r->name = name2;
+	r->striprealm = 1;
+#ifdef WITH_PROXY
 	r->auth_pool = auth_pool;
 	r->acct_pool = acct_pool;
-	r->striprealm = 1;
-
+	
 	if (auth_pool_name &&
 	    (auth_pool_name == acct_pool_name)) { /* yes, ptr comparison */
 		cf_log_info(cs, "\tpool = %s", auth_pool_name);
@@ -1202,6 +1441,7 @@ static int realm_add(realm_config_t *rc, CONF_SECTION *cs)
 		if (auth_pool_name) cf_log_info(cs, "\tauth_pool = %s", auth_pool_name);
 		if (acct_pool_name) cf_log_info(cs, "\tacct_pool = %s", acct_pool_name);
 	}
+#endif
 
 	cp = cf_pair_find(cs, "nostrip");
 	if (cp && (cf_pair_value(cp) == NULL)) {
@@ -1231,6 +1471,29 @@ static int realm_add(realm_config_t *rc, CONF_SECTION *cs)
 		goto error;
 	}
 
+#ifdef HAVE_REGEX_H
+	/*
+	 *	It's a regex.  Add it to a separate list.
+	 */
+	if (name2[0] == '~') {
+		realm_regex_t *rr, **last;
+
+		rr = rad_malloc(sizeof(*rr));
+		
+		last = &realms_regex;
+		while (*last) last = &((*last)->next);  /* O(N^2)... sue me. */
+
+		r->name = name2;
+		rr->realm = r;
+		rr->next = NULL;
+
+		*last = rr;
+
+		cf_log_info(cs, " }");
+		return 1;
+	}
+#endif
+
 	if (!rbtree_insert(realms_byname, r)) {
 		rad_assert("Internal sanity check failed");
 		goto error;
@@ -1260,6 +1523,7 @@ int realms_init(CONF_SECTION *config)
 		return 0;
 	}
 
+#ifdef WITH_PROXY
 	home_servers_byaddr = rbtree_create(home_server_addr_cmp, free, 0);
 	if (!home_servers_byaddr) {
 		realms_free();
@@ -1272,16 +1536,26 @@ int realms_init(CONF_SECTION *config)
 		return 0;
 	}
 
+#ifdef WITH_STATS
+	home_servers_bynumber = rbtree_create(home_server_number_cmp, NULL, 0);
+	if (!home_servers_bynumber) {
+		realms_free();
+		return 0;
+	}
+#endif
+
 	home_pools_byname = rbtree_create(home_pool_name_cmp, free, 0);
 	if (!home_pools_byname) {
 		realms_free();
 		return 0;
 	}
+#endif
 
 	rc = rad_malloc(sizeof(*rc));
 	memset(rc, 0, sizeof(*rc));
 	rc->cs = config;
 
+#ifdef WITH_PROXY
 	cs = cf_subsection_find_next(config, NULL, "proxy");
 	if (cs) {
 		cf_section_parse(cs, rc, proxy_config);
@@ -1292,6 +1566,7 @@ int realms_init(CONF_SECTION *config)
 		rc->fallback = 0;
 		rc->wake_all_if_all_dead= 0;
 	}
+#endif
 
 	for (cs = cf_subsection_find_next(config, NULL, "realm");
 	     cs != NULL;
@@ -1303,8 +1578,10 @@ int realms_init(CONF_SECTION *config)
 		}
 	}
 
+#ifdef WITH_PROXY
 	xlat_register("home_server", xlat_home_server, NULL);
 	xlat_register("home_server_pool", xlat_server_pool, NULL);
+#endif
 
 	/*
 	 *	Swap pointers atomically.
@@ -1316,6 +1593,39 @@ int realms_init(CONF_SECTION *config)
 	return 1;
 }
 
+/*
+ *	Find a realm where "name" might be the regex.
+ */
+REALM *realm_find2(const char *name)
+{
+	REALM myrealm;
+	REALM *realm;
+	
+	if (!name) name = "NULL";
+
+	myrealm.name = name;
+	realm = rbtree_finddata(realms_byname, &myrealm);
+	if (realm) return realm;
+
+#ifdef HAVE_REGEX_H
+	if (realms_regex) {
+		realm_regex_t *this;
+
+		for (this = realms_regex; this != NULL; this = this->next) {
+			if (strcmp(this->realm->name, name) == 0) {
+				return this->realm;
+			}
+		}
+	}
+#endif
+
+	/*
+	 *	Couldn't find a realm.  Look for DEFAULT.
+	 */
+	myrealm.name = "DEFAULT";
+	return rbtree_finddata(realms_byname, &myrealm);
+}
+
 
 /*
  *	Find a realm in the REALM list.
@@ -1331,6 +1641,30 @@ REALM *realm_find(const char *name)
 	realm = rbtree_finddata(realms_byname, &myrealm);
 	if (realm) return realm;
 
+#ifdef HAVE_REGEX_H
+	if (realms_regex) {
+		realm_regex_t *this;
+
+		for (this = realms_regex; this != NULL; this = this->next) {
+			int compare;
+			regex_t reg;
+
+			/*
+			 *	Include substring matches.
+			 */
+			if (regcomp(&reg, this->realm->name + 1,
+				    REG_EXTENDED | REG_NOSUB | REG_ICASE) != 0) {
+				continue;
+			}
+
+			compare = regexec(&reg, name, 0, NULL, 0);
+			regfree(&reg);
+
+			if (compare == 0) return this->realm;
+		}
+	}
+#endif
+
 	/*
 	 *	Couldn't find a realm.  Look for DEFAULT.
 	 */
@@ -1339,6 +1673,7 @@ REALM *realm_find(const char *name)
 }
 
 
+#ifdef WITH_PROXY
 home_server *home_server_ldb(const char *realmname,
 			     home_pool_t *pool, REQUEST *request)
 {
@@ -1440,7 +1775,12 @@ home_server *home_server_ldb(const char *realmname,
 			return home;
 		}
 
-		DEBUG3("PROXY %s %d\t%s %d",
+		if (!found) {
+			found = home;
+			continue;
+		}
+
+		RDEBUG3("PROXY %s %d\t%s %d",
 		       found->name, found->currently_outstanding,
 		       home->name, home->currently_outstanding);
 
@@ -1449,7 +1789,7 @@ home_server *home_server_ldb(const char *realmname,
 		 *	one we previously found.
 		 */
 		if (home->currently_outstanding < found->currently_outstanding) {
-			DEBUG3("Choosing %s: It's less busy than %s",
+			RDEBUG3("PROXY Choosing %s: It's less busy than %s",
 			       home->name, found->name);
 			found = home;
 			continue;
@@ -1460,20 +1800,7 @@ home_server *home_server_ldb(const char *realmname,
 		 *	we found.
 		 */
 		if (home->currently_outstanding > found->currently_outstanding) {
-			DEBUG3("Skipping %s: It's busier than %s",
-			       home->name, found->name);
-			continue;
-		}
-
-		if (home->total_requests_sent < found->total_requests_sent) {
-			DEBUG3("Choosing %s: It's been less busy than %s",
-			       home->name, found->name);
-			found = home;
-			continue;
-		}
-
-		if (home->total_requests_sent > found->total_requests_sent) {
-			DEBUG3("Skipping %s: It's been busier than %s",
+			RDEBUG3("PROXY Skipping %s: It's busier than %s",
 			       home->name, found->name);
 			continue;
 		}
@@ -1490,6 +1817,13 @@ home_server *home_server_ldb(const char *realmname,
 
 	if (found) return found;
 
+	/*
+	 *	There's a fallback if they're all dead.
+	 */
+	if (pool->fallback) {
+		return pool->fallback;
+	}
+
 	/*
 	 *	No live match found, and no fallback to the "DEFAULT"
 	 *	realm.  We fix this by blindly marking all servers as
@@ -1535,7 +1869,7 @@ home_server *home_server_ldb(const char *realmname,
 		}
 		if (!pool) return NULL;
 
-		DEBUG2("  Realm %s has no live home servers.  Falling back to the DEFAULT realm.", realmname);
+		RDEBUG2("PROXY - realm %s has no live home servers.  Falling back to the DEFAULT realm.", realmname);
 		return home_server_ldb(rd->name, pool, request);
 	}
 
@@ -1550,8 +1884,24 @@ home_server *home_server_find(fr_ipaddr_t *ipaddr, int port)
 {
 	home_server myhome;
 
+	memset(&myhome, 0, sizeof(myhome));
 	myhome.ipaddr = *ipaddr;
 	myhome.port = port;
+	myhome.server = NULL;	/* we're not called for internal proxying */
 
 	return rbtree_finddata(home_servers_byaddr, &myhome);
 }
+
+#ifdef WITH_STATS
+home_server *home_server_bynumber(int number)
+{
+	home_server myhome;
+
+	memset(&myhome, 0, sizeof(myhome));
+	myhome.number = number;
+	myhome.server = NULL;	/* we're not called for internal proxying */
+
+	return rbtree_finddata(home_servers_bynumber, &myhome);
+}
+#endif
+#endif
