@@ -2,7 +2,7 @@
  * valuepair.c	Valuepair functions that are radiusd-specific
  *		and as such do not belong in the library.
  *
- * Version:	$Id: valuepair.c,v 1.91 2008/05/12 14:41:32 aland Exp $
+ * Version:	$Id$
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: valuepair.c,v 1.91 2008/05/12 14:41:32 aland Exp $")
+RCSID("$Id$")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/rad_assert.h>
@@ -80,8 +80,15 @@ int radius_compare_vps(REQUEST *request, VALUE_PAIR *check, VALUE_PAIR *vp)
 		/*
 		 *	Include substring matches.
 		 */
-		regcomp(&reg, (char *)check->vp_strvalue,
-			REG_EXTENDED);
+		compare = regcomp(&reg, check->vp_strvalue, REG_EXTENDED);
+		if (compare != 0) {
+			char buffer[256];
+			regerror(compare, &reg, buffer, sizeof(buffer));
+
+			RDEBUG("Invalid regular expression %s: %s",
+			       check->vp_strvalue, buffer);
+			return -1;
+		}
 		compare = regexec(&reg, value,  REQUEST_MAX_REGEX + 1,
 				  rxmatch, 0);
 		regfree(&reg);
@@ -150,8 +157,16 @@ int radius_compare_vps(REQUEST *request, VALUE_PAIR *check, VALUE_PAIR *vp)
 		/*
 		 *	Include substring matches.
 		 */
-		regcomp(&reg, (char *)check->vp_strvalue,
-			REG_EXTENDED);
+		compare = regcomp(&reg, (char *)check->vp_strvalue,
+				  REG_EXTENDED);
+		if (compare != 0) {
+			char buffer[256];
+			regerror(compare, &reg, buffer, sizeof(buffer));
+
+			RDEBUG("Invalid regular expression %s: %s",
+			       check->vp_strvalue, buffer);
+			return -1;
+		}
 		compare = regexec(&reg, value,  REQUEST_MAX_REGEX + 1,
 				  rxmatch, 0);
 		regfree(&reg);
@@ -736,4 +751,5 @@ void debug_pair_list(VALUE_PAIR *vp)
 		fputc('\n', fr_log_fp);
 		vp = vp->next;
 	}
+	fflush(fr_log_fp);
 }
