@@ -1,7 +1,7 @@
 /*
  * rlm_eap_tls.c  contains the interfaces that are called from eap
  *
- * Version:     $Id: rlm_eap_tls.c,v 1.54 2008/03/22 08:31:03 aland Exp $
+ * Version:     $Id$
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: rlm_eap_tls.c,v 1.54 2008/03/22 08:31:03 aland Exp $")
+RCSID("$Id$")
 
 #include <freeradius-devel/autoconf.h>
 
@@ -32,12 +32,29 @@ RCSID("$Id: rlm_eap_tls.c,v 1.54 2008/03/22 08:31:03 aland Exp $")
 #include <openssl/rand.h>
 #endif
 
+#ifdef HAVE_OPENSSL_EVP_H
+#include <openssl/evp.h>
+#endif
+
 #include "rlm_eap_tls.h"
+#include "config.h"
 
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
 
+static CONF_PARSER cache_config[] = {
+	{ "enable", PW_TYPE_BOOLEAN,
+	  offsetof(EAP_TLS_CONF, session_cache_enable), NULL, "no" },
+	{ "lifetime", PW_TYPE_INTEGER,
+	  offsetof(EAP_TLS_CONF, session_timeout), NULL, "24" },
+	{ "max_entries", PW_TYPE_INTEGER,
+	  offsetof(EAP_TLS_CONF, session_cache_size), NULL, "255" },
+	{ "name", PW_TYPE_STRING_PTR,
+	  offsetof(EAP_TLS_CONF, session_id_name), NULL, NULL},
+ 	{ NULL, -1, 0, NULL, NULL }           /* end the list */
+};
+
 static CONF_PARSER module_config[] = {
 	{ "rsa_key_exchange", PW_TYPE_BOOLEAN,
 	  offsetof(EAP_TLS_CONF, rsa_key), NULL, "no" },
@@ -80,6 +97,8 @@ static CONF_PARSER module_config[] = {
 	{ "make_cert_command", PW_TYPE_STRING_PTR,
 	  offsetof(EAP_TLS_CONF, make_cert_command), NULL, NULL},
 
+	{ "cache", PW_TYPE_SUBSECTION, 0, NULL, (const void *) cache_config },
+
  	{ NULL, -1, 0, NULL, NULL }           /* end the list */
 };
 
@@ -115,6 +134,85 @@ static int load_dh_params(SSL_CTX *ctx, char *file)
 	return 0;
 }
 
+
+/*
+ *	Generate ephemeral RSA keys.
+ */
+static int generate_eph_rsa_key(SSL_CTX *ctx)
+{
+	RSA *rsa;
+
+	rsa = RSA_generate_key(512, RSA_F4, NULL, NULL);
+
+	if (!SSL_CTX_set_tmp_rsa(ctx, rsa)) {
+		radlog(L_ERR, "rlm_eap_tls: Couldn't set ephemeral RSA key");
+		return -1;
+	}
+
+	RSA_free(rsa);
+	return 0;
+}
+
+
+/*
+ *	These functions don't do anything other than print debugging
+ *	messages.
+ *
+ *	FIXME: Write sessions to some long-term storage, so that
+ *	       session resumption can still occur after the server
+ *	       restarts.
+ */
+#define MAX_SESSION_SIZE (256)
+
+static void cbtls_remove_session(UNUSED SSL_CTX *ctx, SSL_SESSION *sess)
+{
+	size_t size;
+	char buffer[2 * MAX_SESSION_SIZE + 1];
+
+	size = sess->session_id_length;
+	if (size > MAX_SESSION_SIZE) size = MAX_SESSION_SIZE;
+
+	fr_bin2hex(sess->session_id, buffer, size);
+
+        DEBUG2("  SSL: Removing session %s from the cache", buffer);
+        SSL_SESSION_free(sess);
+
+        return;
+}
+
+static int cbtls_new_session(UNUSED SSL *s, SSL_SESSION *sess)
+{
+	size_t size;
+	char buffer[2 * MAX_SESSION_SIZE + 1];
+
+	size = sess->session_id_length;
+	if (size > MAX_SESSION_SIZE) size = MAX_SESSION_SIZE;
+
+	fr_bin2hex(sess->session_id, buffer, size);
+
+	DEBUG2("  SSL: adding session %s to cache", buffer);
+
+	return 1;
+}
+
+static SSL_SESSION *cbtls_get_session(UNUSED SSL *s,
+				      unsigned char *data, int len,
+				      UNUSED int *copy)
+{
+	size_t size;
+	char buffer[2 * MAX_SESSION_SIZE + 1];
+
+	size = len;
+	if (size > MAX_SESSION_SIZE) size = MAX_SESSION_SIZE;
+
+	fr_bin2hex(data, buffer, size);
+
+        DEBUG2("  SSL: Client requested nonexistent cached session %s",
+	       buffer);
+
+	return NULL;
+}
+
 /*
  *	Before trusting a certificate, you must make sure that the
  *	certificate is 'valid'. There are several steps that your
@@ -152,6 +250,7 @@ static int cbtls_verify(int ok, X509_STORE_CTX *ctx)
 	int err, depth;
 	EAP_TLS_CONF *conf;
 	int my_ok = ok;
+	REQUEST *request;
 
 	client_cert = X509_STORE_CTX_get_current_cert(ctx);
 	err = X509_STORE_CTX_get_error(ctx);
@@ -169,6 +268,7 @@ static int cbtls_verify(int ok, X509_STORE_CTX *ctx)
 	 */
 	ssl = X509_STORE_CTX_get_ex_data(ctx, SSL_get_ex_data_X509_STORE_CTX_idx());
 	handler = (EAP_HANDLER *)SSL_get_ex_data(ssl, 0);
+	request = handler->request;
 	conf = (EAP_TLS_CONF *)SSL_get_ex_data(ssl, 1);
 
 	/*
@@ -239,7 +339,7 @@ static int cbtls_verify(int ok, X509_STORE_CTX *ctx)
 				/* if this fails, fail the verification */
 				my_ok = 0;
 			} else {
-				DEBUG2("    rlm_eap_tls: checking certificate CN (%s) with xlat'ed value (%s)", common_name, cn_str);
+				RDEBUG2("checking certificate CN (%s) with xlat'ed value (%s)", common_name, cn_str);
 				if (strcmp(cn_str, common_name) != 0) {
 					radlog(L_AUTH, "rlm_eap_tls: Certificate CN (%s) does not match specified value (%s)!", common_name, cn_str);
 					my_ok = 0;
@@ -249,19 +349,33 @@ static int cbtls_verify(int ok, X509_STORE_CTX *ctx)
 	} /* depth == 0 */
 
 	if (debug_flag > 0) {
-		DEBUG2("chain-depth=%d, ", depth);
-		DEBUG2("error=%d", err);
+		RDEBUG2("chain-depth=%d, ", depth);
+		RDEBUG2("error=%d", err);
 
-		DEBUG2("--> User-Name = %s", handler->identity);
-		DEBUG2("--> BUF-Name = %s", common_name);
-		DEBUG2("--> subject = %s", subject);
-		DEBUG2("--> issuer  = %s", issuer);
-		DEBUG2("--> verify return:%d", my_ok);
+		RDEBUG2("--> User-Name = %s", handler->identity);
+		RDEBUG2("--> BUF-Name = %s", common_name);
+		RDEBUG2("--> subject = %s", subject);
+		RDEBUG2("--> issuer  = %s", issuer);
+		RDEBUG2("--> verify return:%d", my_ok);
 	}
 	return my_ok;
 }
 
 
+/*
+ *	Free cached session data, which is always a list of VALUE_PAIRs
+ */
+static void eaptls_session_free(UNUSED void *parent, void *data_ptr,
+				UNUSED CRYPTO_EX_DATA *ad, UNUSED int idx,
+				UNUSED long argl, UNUSED void *argp)
+{
+	VALUE_PAIR *vp = data_ptr;
+	if (!data_ptr) return;
+
+	pairfree(&vp);
+}
+
+
 /*
  *	Create Global context SSL and use it in every new session
  *
@@ -285,6 +399,15 @@ static SSL_CTX *init_tls_ctx(EAP_TLS_CONF *conf)
 	SSL_library_init();
 	SSL_load_error_strings();
 
+	/*
+	 *	SHA256 is in all versions of OpenSSL, but isn't
+	 *	initialized by default.  It's needed for WiMAX
+	 *	certificates.
+	 */
+#ifdef HAVE_OPENSSL_EVP_SHA256
+	EVP_add_digest(EVP_sha256());
+#endif
+
 	meth = TLSv1_method();
 	ctx = SSL_CTX_new(meth);
 
@@ -328,10 +451,12 @@ static SSL_CTX *init_tls_ctx(EAP_TLS_CONF *conf)
 	}
 
 	/* Load the CAs we trust */
-	if (!SSL_CTX_load_verify_locations(ctx, conf->ca_file, conf->ca_path)) {
-		radlog(L_ERR, "rlm_eap: SSL error %s", ERR_error_string(ERR_get_error(), NULL));
-		radlog(L_ERR, "rlm_eap_tls: Error reading Trusted root CA list %s",conf->ca_file );
-		return NULL;
+	if (conf->ca_file || conf->ca_path) {
+		if (!SSL_CTX_load_verify_locations(ctx, conf->ca_file, conf->ca_path)) {
+			radlog(L_ERR, "rlm_eap: SSL error %s", ERR_error_string(ERR_get_error(), NULL));
+			radlog(L_ERR, "rlm_eap_tls: Error reading Trusted root CA list %s",conf->ca_file );
+			return NULL;
+		}
 	}
 	if (conf->ca_file && *conf->ca_file) SSL_CTX_set_client_CA_list(ctx, SSL_load_client_CA_file(conf->ca_file));
 	if (!(SSL_CTX_use_PrivateKey_file(ctx, conf->private_key_file, type))) {
@@ -392,6 +517,17 @@ static SSL_CTX *init_tls_ctx(EAP_TLS_CONF *conf)
 	/* Set Info callback */
 	SSL_CTX_set_info_callback(ctx, cbtls_info);
 
+	/*
+	 *	Callbacks, etc. for session resumption.
+	 */						      
+	if (conf->session_cache_enable) {
+		SSL_CTX_sess_set_new_cb(ctx, cbtls_new_session);
+		SSL_CTX_sess_set_get_cb(ctx, cbtls_get_session);
+		SSL_CTX_sess_set_remove_cb(ctx, cbtls_remove_session);
+
+		SSL_CTX_set_quiet_shutdown(ctx, 1);
+	}
+
 	/*
 	 *	Check the certificates for revocation.
 	 */
@@ -437,6 +573,70 @@ static SSL_CTX *init_tls_ctx(EAP_TLS_CONF *conf)
 		}
 	}
 
+	/*
+	 *	Setup session caching
+	 */
+	if (conf->session_cache_enable) {
+		/*
+		 *	Create a unique context Id per EAP-TLS configuration.
+		 */
+		if (conf->session_id_name) {
+			snprintf(conf->session_context_id,
+				 sizeof(conf->session_context_id),
+				 "FreeRADIUS EAP-TLS %s",
+				 conf->session_id_name);
+		} else {
+			snprintf(conf->session_context_id,
+				 sizeof(conf->session_context_id),
+				 "FreeRADIUS EAP-TLS %p", conf);
+		}
+
+		/*
+		 *	Cache it, and DON'T auto-clear it.
+		 */
+		SSL_CTX_set_session_cache_mode(ctx, SSL_SESS_CACHE_SERVER | SSL_SESS_CACHE_NO_AUTO_CLEAR);
+					       
+		SSL_CTX_set_session_id_context(ctx,
+					       (unsigned char *) conf->session_context_id,
+					       (unsigned int) strlen(conf->session_context_id));
+
+		/*
+		 *	Our timeout is in hours, this is in seconds.
+		 */
+		SSL_CTX_set_timeout(ctx, conf->session_timeout * 3600);
+		
+		/*
+		 *	Set the maximum number of entries in the
+		 *	session cache.
+		 */
+		SSL_CTX_sess_set_cache_size(ctx, conf->session_cache_size);
+
+	} else {
+		SSL_CTX_set_session_cache_mode(ctx, SSL_SESS_CACHE_OFF);
+	}
+
+	/*
+	 *	Register the application indices.  We can't use
+	 *	hard-coded "0" and "1" as before, because we need to
+	 *	set up a "free" handler for the cached session
+	 *	information.
+	 */
+	if (eaptls_handle_idx < 0) {
+		eaptls_handle_idx = SSL_get_ex_new_index(0, "eaptls_handle_idx",
+							  NULL, NULL, NULL);
+	}
+	
+	if (eaptls_conf_idx < 0) {
+		eaptls_conf_idx = SSL_get_ex_new_index(0, "eaptls_conf_idx",
+							  NULL, NULL, NULL);
+	}
+
+	if (eaptls_session_idx < 0) {
+		eaptls_session_idx = SSL_get_ex_new_index(0, "eaptls_session_idx",
+							  NULL, NULL,
+							  eaptls_session_free);
+	}
+
 	return ctx;
 }
 
@@ -564,6 +764,10 @@ static int eaptls_attach(CONF_SECTION *cs, void **instance)
 		return -1;
 	}
 
+        if (generate_eph_rsa_key(inst->ctx) < 0) {
+                return -1;
+        }
+
 	*instance = inst;
 
 	return 0;
@@ -597,9 +801,26 @@ static int eaptls_initiate(void *type_arg, EAP_HANDLER *handler)
 	VALUE_PAIR	*vp;
 	int		client_cert = TRUE;
 	int		verify_mode = 0;
+	REQUEST		*request = handler->request;
 
 	inst = (eap_tls_t *)type_arg;
 
+	/*
+	 *	Manually flush the sessions every so often.  If HALF
+	 *	of the session lifetime has passed since we last
+	 *	flushed, then flush it again.
+	 *
+	 *	FIXME: Also do it every N sessions?
+	 */
+	if (inst->conf->session_cache_enable &&
+	    ((inst->conf->session_last_flushed + (inst->conf->session_timeout * 1800)) <= request->timestamp)) {
+		RDEBUG2("Flushing SSL sessions (of #%ld)",
+			SSL_CTX_sess_number(inst->ctx));
+
+		SSL_CTX_flush_sessions(inst->ctx, request->timestamp);
+		inst->conf->session_last_flushed = request->timestamp;
+	}
+
 	/*
 	 *	If we're TTLS or PEAP, then do NOT require a client
 	 *	certificate.
@@ -632,7 +853,7 @@ static int eaptls_initiate(void *type_arg, EAP_HANDLER *handler)
 	 *	Verify the peer certificate, if asked.
 	 */
 	if (client_cert) {
-		DEBUG2(" rlm_eap_tls: Requiring client certificate");
+		RDEBUG2("Requiring client certificate");
 		verify_mode = SSL_VERIFY_PEER;
 		verify_mode |= SSL_VERIFY_FAIL_IF_NO_PEER_CERT;
 		verify_mode |= SSL_VERIFY_CLIENT_ONCE;
@@ -685,12 +906,25 @@ static int eaptls_initiate(void *type_arg, EAP_HANDLER *handler)
 	handler->opaque = ((void *)ssn);
 	handler->free_opaque = session_free;
 
-	DEBUG2("  rlm_eap_tls: Initiate");
+	RDEBUG2("Initiate");
 
 	/*
-	 *	PEAP-specific breakage.
+	 *	Set up type-specific information.
 	 */
-	if (handler->eap_type == PW_EAP_PEAP) {
+	switch (handler->eap_type) {
+	case PW_EAP_TLS:
+	default:
+		ssn->prf_label = "client EAP encryption";
+		break;
+
+	case PW_EAP_TTLS:
+		ssn->prf_label = "ttls keying material";
+		break;
+
+		/*
+		 *	PEAP-specific breakage.
+		 */
+	case PW_EAP_PEAP:
 		/*
 		 *	As it is a poorly designed protocol, PEAP uses
 		 *	bits in the TLS header to indicate PEAP
@@ -708,6 +942,13 @@ static int eaptls_initiate(void *type_arg, EAP_HANDLER *handler)
 		 *	we force it here.
 		 */
 		ssn->length_flag = 0;
+
+		ssn->prf_label = "client EAP encryption";
+		break;
+	}
+
+	if (inst->conf->session_cache_enable) {
+		ssn->allow_session_resumption = 1; /* otherwise it's zero */
 	}
 
 	/*
@@ -715,7 +956,7 @@ static int eaptls_initiate(void *type_arg, EAP_HANDLER *handler)
 	 *	related handshaking or application data.
 	 */
 	status = eaptls_start(handler->eap_ds, ssn->peap_flag);
-	DEBUG2("  rlm_eap_tls: Start returned %d", status);
+	RDEBUG2("Start returned %d", status);
 	if (status == 0)
 		return 0;
 
@@ -730,15 +971,17 @@ static int eaptls_initiate(void *type_arg, EAP_HANDLER *handler)
 /*
  *	Do authentication, by letting EAP-TLS do most of the work.
  */
-static int eaptls_authenticate(void *arg UNUSED, EAP_HANDLER *handler)
+static int eaptls_authenticate(void *arg, EAP_HANDLER *handler)
 {
 	eaptls_status_t	status;
 	tls_session_t *tls_session = (tls_session_t *) handler->opaque;
+	REQUEST *request = handler->request;
+	eap_tls_t *inst = (eap_tls_t *) arg;
 
-	DEBUG2("  rlm_eap_tls: Authenticate");
+	RDEBUG2("Authenticate");
 
 	status = eaptls_process(handler);
-	DEBUG2("  eaptls_process returned %d\n", status);
+	RDEBUG2("eaptls_process returned %d\n", status);
 	switch (status) {
 		/*
 		 *	EAP-TLS handshake was successful, return an
@@ -760,7 +1003,7 @@ static int eaptls_authenticate(void *arg UNUSED, EAP_HANDLER *handler)
 		 *	data.
 		 */
 	case EAPTLS_OK:
-		DEBUG2("  rlm_eap_tls: Received unexpected tunneled data after successful handshake.");
+		RDEBUG2("Received unexpected tunneled data after successful handshake.");
 #ifndef NDEBUG
 		if ((debug_flag > 2) && fr_log_fp) {
 			unsigned int i;
@@ -780,25 +1023,62 @@ static int eaptls_authenticate(void *arg UNUSED, EAP_HANDLER *handler)
 		}
 #endif
 
-		eaptls_fail(handler->eap_ds, 0);
+		eaptls_fail(handler, 0);
 		return 0;
 		break;
 
 		/*
 		 *	Anything else: fail.
+		 *
+		 *	Also, remove the session from the cache so that
+		 *	the client can't re-use it.
 		 */
 	default:
+		if (inst->conf->session_cache_enable) {	
+			SSL_CTX_remove_session(inst->ctx,
+					       tls_session->ssl->session);
+		}
+
 		return 0;
 	}
 
 	/*
-	 *	Success: Return MPPE keys.
+	 *	New sessions cause some additional information to be
+	 *	cached.
 	 */
-	eaptls_success(handler->eap_ds, 0);
-	eaptls_gen_mppe_keys(&handler->request->reply->vps,
-			     tls_session->ssl,
-			     "client EAP encryption");
-	return 1;
+	if (!SSL_session_reused(tls_session->ssl)) {
+		/*
+		 *	FIXME: Store miscellaneous data.
+		 */
+		RDEBUG2("Adding user data to cached session");
+		
+#if 0
+		SSL_SESSION_set_ex_data(tls_session->ssl->session,
+					ssl_session_idx_user_session, session_data);
+#endif
+	} else {
+		/*
+		 *	FIXME: Retrieve miscellaneous data.
+		 */
+#if 0
+		data = SSL_SESSION_get_ex_data(tls_session->ssl->session,
+					       ssl_session_idx_user_session);
+
+		if (!session_data) {
+			radlog_request(L_ERR, 0, request,
+				       "No user session data in cached session - "
+				       " REJECTING");
+			return 0;
+		}
+#endif
+
+		RDEBUG2("Retrieved session data from cached session");
+	}
+
+	/*
+	 *	Success: Automatically return MPPE keys.
+	 */
+	return eaptls_success(handler, 0);
 }
 
 /*
