@@ -2,7 +2,7 @@
  *  sql.c		rlm_sql - FreeRADIUS SQL Module
  *		Main code directly taken from ICRADIUS
  *
- * Version:	$Id: sql.c,v 1.98 2008/05/09 07:29:38 aland Exp $
+ * Version:	$Id$
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: sql.c,v 1.98 2008/05/09 07:29:38 aland Exp $")
+RCSID("$Id$")
 
 #include	<freeradius-devel/radiusd.h>
 
@@ -404,7 +404,7 @@ int sql_userparse(VALUE_PAIR ** first_pair, SQL_ROW row)
 	 */
 	pair = pairmake(row[2], value, operator);
 	if (pair == NULL) {
-		radlog(L_ERR, "rlm_sql: Failed to create the pair: %s", librad_errstr);
+		radlog(L_ERR, "rlm_sql: Failed to create the pair: %s", fr_strerror());
 		return -1;
 	}
 	if (do_xlat) {
