@@ -3,7 +3,7 @@
  *		accounting:     Functions to write radwtmp file.
  *		Also contains handler for "Group".
  *
- * Version:	$Id: rlm_unix.c,v 1.85 2007/12/10 16:07:30 aland Exp $
+ * Version:	$Id$
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
  */
 
 #include	<freeradius-devel/ident.h>
-RCSID("$Id: rlm_unix.c,v 1.85 2007/12/10 16:07:30 aland Exp $")
+RCSID("$Id$")
 
 #include	<freeradius-devel/radiusd.h>
 
@@ -249,7 +249,8 @@ static int unix_getpw(UNUSED void *instance, REQUEST *request,
 	 *	Users with a particular shell are denied access
 	 */
 	if (strcmp(pwd->pw_shell, DENY_SHELL) == 0) {
-		radlog(L_AUTH, "rlm_unix: [%s]: invalid shell", name);
+		radlog_request(L_AUTH, 0, request,
+			       "rlm_unix: [%s]: invalid shell", name);
 		return RLM_MODULE_REJECT;
 	}
 #endif
@@ -267,7 +268,7 @@ static int unix_getpw(UNUSED void *instance, REQUEST *request,
 	}
 	endusershell();
 	if (shell == NULL) {
-		radlog(L_AUTH, "rlm_unix: [%s]: invalid shell [%s]",
+		radlog_request(L_AUTH, 0, request, "[%s]: invalid shell [%s]",
 		       name, pwd->pw_shell);
 		return RLM_MODULE_REJECT;
 	}
@@ -280,7 +281,7 @@ static int unix_getpw(UNUSED void *instance, REQUEST *request,
 	 */
 	if (spwd && spwd->sp_expire > 0 &&
 	    (request->timestamp / 86400) > spwd->sp_expire) {
-		radlog(L_AUTH, "rlm_unix: [%s]: password has expired", name);
+		radlog_request(L_AUTH, 0, request, "[%s]: password has expired", name);
 		return RLM_MODULE_REJECT;
 	}
 #endif
@@ -291,7 +292,7 @@ static int unix_getpw(UNUSED void *instance, REQUEST *request,
 	 */
 	if ((pwd->pw_expire > 0) &&
 	    (request->timestamp > pwd->pw_expire)) {
-		radlog(L_AUTH, "rlm_unix: [%s]: password has expired", name);
+		radlog_request(L_AUTH, 0, request, "[%s]: password has expired", name);
 		return RLM_MODULE_REJECT;
 	}
 #endif
@@ -354,7 +355,7 @@ static int unix_authenticate(void *instance, REQUEST *request)
 
 	if (!request->password ||
 	    (request->password->attribute != PW_USER_PASSWORD)) {
-		radlog(L_AUTH, "rlm_unix: Attribute \"User-Password\" is required for authentication.");
+		radlog_request(L_AUTH, 0, request, "Attribute \"User-Password\" is required for authentication.");
 		return RLM_MODULE_INVALID;
 	}
 
@@ -366,8 +367,8 @@ static int unix_authenticate(void *instance, REQUEST *request)
 	 */
 	if (fr_crypt_check((char *) request->password->vp_strvalue,
 			     (char *) vp->vp_strvalue) != 0) {
-		radlog(L_AUTH, "rlm_unix: [%s]: invalid password",
-		       request->username->vp_strvalue);
+		radlog_request(L_AUTH, 0, request, "invalid password \"%s\"",
+			       request->username->vp_strvalue);
 		return RLM_MODULE_REJECT;
 	}
 #endif /* OSFFIA */
@@ -430,12 +431,12 @@ static int unix_accounting(void *instance, REQUEST *request)
 	 *	No radwtmp.  Don't do anything.
 	 */
 	if (!inst->radwtmp) {
-		DEBUG2("rlm_unix: No radwtmp file configured.  Ignoring accounting request.");
+		RDEBUG2("No radwtmp file configured.  Ignoring accounting request.");
 		return RLM_MODULE_NOOP;
 	}
 
 	if (request->packet->src_ipaddr.af != AF_INET) {
-		DEBUG2("rlm_unix: IPv6 is not supported!");
+		RDEBUG2("IPv6 is not supported!");
 		return RLM_MODULE_NOOP;
 	}
 
@@ -515,9 +516,8 @@ static int unix_accounting(void *instance, REQUEST *request)
 	 */
 	if (nas_address == 0) {
 		nas_address = request->packet->src_ipaddr.ipaddr.ip4addr.s_addr;
-
-		s = request->client->shortname;
 	}
+	s = request->client->shortname;
 	if (!s || s[0] == 0) s = uue(&(nas_address));
 
 #ifdef __linux__
